@@ -1,7 +1,7 @@
 /*
  *      ao_tasks.c - this file is part of Addons, a Geany plugin
  *
- *      Copyright 2009-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2009-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -25,7 +25,10 @@
 #include <gtk/gtk.h>
 #include <glib-object.h>
 
-#include "geanyplugin.h"
+#ifdef HAVE_CONFIG_H
+	#include "config.h"
+#endif
+#include <geanyplugin.h>
 
 #include "addons.h"
 #include "ao_tasks.h"
@@ -93,7 +96,7 @@ static void ao_tasks_finalize  			(GObject *object);
 static void ao_tasks_show				(AoTasks *t);
 static void ao_tasks_hide				(AoTasks *t);
 
-G_DEFINE_TYPE(AoTasks, ao_tasks, G_TYPE_OBJECT);
+G_DEFINE_TYPE(AoTasks, ao_tasks, G_TYPE_OBJECT)
 
 
 static void ao_tasks_set_property(GObject *object, guint prop_id,
@@ -112,6 +115,8 @@ static void ao_tasks_set_property(GObject *object, guint prop_id,
 				ao_tasks_hide(AO_TASKS(object));
 
 			priv->enable_tasks = new_val;
+			if (priv->enable_tasks && main_is_realized() && ! priv->active)
+				ao_tasks_set_active(AO_TASKS(object));
 			break;
 		}
 		case PROP_SCAN_ALL_DOCUMENTS:
@@ -289,7 +294,7 @@ static void ao_tasks_hide(AoTasks *t)
 	}
 	if (priv->popup_menu)
 	{
-		gtk_widget_destroy(priv->popup_menu);
+		g_object_unref(priv->popup_menu);
 		priv->popup_menu = NULL;
 	}
 }
@@ -473,6 +478,7 @@ static void ao_tasks_show(AoTasks *t)
 		gtk_label_new(_("Tasks")));
 
 	priv->popup_menu = create_popup_menu(t);
+	g_object_ref_sink(priv->popup_menu);
 }
 
 
@@ -482,8 +488,7 @@ void ao_tasks_activate(AoTasks *t)
 
 	if (priv->enable_tasks)
 	{
-		GtkNotebook *notebook = GTK_NOTEBOOK(
-			ui_lookup_widget(geany->main_widgets->window, "notebook_info"));
+		GtkNotebook *notebook = GTK_NOTEBOOK(geany->main_widgets->message_window_notebook);
 		gint page_number = gtk_notebook_page_num(notebook, priv->page);
 
 		gtk_notebook_set_current_page(notebook, page_number);
@@ -504,22 +509,23 @@ void ao_tasks_remove(AoTasks *t, GeanyDocument *cur_doc)
 
 	if (gtk_tree_model_get_iter_first(model, &iter))
 	{
-		while (TRUE)
+		gboolean has_next;
+
+		do
 		{
 			gtk_tree_model_get(model, &iter, TLIST_COL_FILENAME, &filename, -1);
 
 			if (utils_str_equal(filename, cur_doc->file_name))
 			{	/* gtk_list_store_remove() manages the iter and set it to the next row */
-				if (! gtk_list_store_remove(priv->store, &iter))
-					break;
+				has_next = gtk_list_store_remove(priv->store, &iter);
 			}
 			else
 			{	/* if we didn't delete the row, we need to manage the iter manually */
-				if (! gtk_tree_model_iter_next(model, &iter))
-					break;
+				has_next = gtk_tree_model_iter_next(model, &iter);
 			}
 			g_free(filename);
 		}
+		while (has_next);
 	}
 }
 
@@ -552,7 +558,7 @@ static void create_task(AoTasks *t, GeanyDocument *doc, gint line, const gchar *
 
 static void update_tasks_for_doc(AoTasks *t, GeanyDocument *doc)
 {
-	guint lines, line;
+	gint lines, line;
 	gchar *line_buf, *display_name, *task_start;
 	gchar **token;
 	AoTasksPrivate *priv = AO_TASKS_GET_PRIVATE(t);
@@ -612,7 +618,7 @@ static gboolean ao_tasks_select_task(GtkTreeModel *model, GtkTreePath *path,
 	AoTasksPrivate *priv = AO_TASKS_GET_PRIVATE(data);
 	gint line, selected_line;
 	gchar *filename = NULL;
-	gchar *selected_filename = NULL;
+	const gchar *selected_filename = NULL;
 	gboolean ret = FALSE;
 
 	if (priv->scan_all_documents)
