@@ -20,6 +20,11 @@
  *      MA 02110-1301, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+	#include "config.h"
+#endif
+#include <geanyplugin.h>
+
 #include "switch_head_impl.h"
 #include "utils.h"
 
@@ -131,7 +136,7 @@ switch_head_impl_cleanup()
 static void
 fill_default_languages_list()
 {
-	Language* lang = g_malloc0(sizeof(Language));
+	Language* lang = NULL;
 
 	languages = NULL;
 
@@ -221,12 +226,12 @@ menu_item_activate(guint key_id)
 	gchar* p_str = NULL;	/* Local variables, used as temporary buffers */
 	gchar* p_str2 = NULL;
 
-	log_func();
-	log_debug("current_doc->file_name == %s", current_doc->file_name);
-	log_debug("geany->documents_array->len == %d", geany->documents_array->len);
-
 	if(current_doc != NULL && current_doc->file_name != NULL && current_doc->file_name[0] != '\0')
 	{
+		log_func();
+		log_debug("current_doc->file_name == %s", current_doc->file_name);
+		log_debug("geany->documents_array->len == %d", geany->documents_array->len);
+
 		/* Get the basename, e.g. : "/home/me/file.cpp" -> "file.cpp" */
 		basename = g_path_get_basename(current_doc->file_name);
 
