@@ -19,7 +19,11 @@
 
 #include <string.h>
 
-#include "geanyplugin.h"
+#ifdef HAVE_CONFIG_H
+	#include "config.h"
+#endif
+#include <geanyplugin.h>
+
 #include "gdb-io-priv.h"
 
 
