@@ -18,6 +18,10 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. *
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
@@ -562,8 +566,8 @@ load_click(GtkWidget * btn, gpointer user_data)
 							{
 								gchar *base_path;
 								gchar *ldd = g_strdup_printf("ldd \"%s\"", fn);
-								FILE *fh = popen(ldd, "r");
-								if (fh)
+								FILE *ph = popen(ldd, "r");
+								if (ph)
 								{
 									ssize_t r = 0;
 									char *buf = NULL;
@@ -571,14 +575,14 @@ load_click(GtkWidget * btn, gpointer user_data)
 									gboolean have_x = FALSE;
 									while (r >= 0)
 									{
-										r = getline(&buf, &len, fh);
+										r = getline(&buf, &len, ph);
 										if (len && buf
 												&& strstr(buf, "libX11.so"))
 										{
 											have_x = TRUE;
 										}
 									}
-									fclose(fh);
+									pclose(ph);
 									gtk_toggle_button_set_active
 											(GTK_TOGGLE_BUTTON(term_chk),
 											!have_x);
@@ -586,11 +590,12 @@ load_click(GtkWidget * btn, gpointer user_data)
 								gdbio_load(fn);
 								if (pipe_chk_active())
 									pipe_click(pipe_chk, NULL);
-									
+
 								/* set working dir after loading */
 								base_path = g_path_get_dirname(fn);
 								gdbio_send_cmd("-environment-cd %s\n", base_path);
 								g_free(base_path);
+								g_free(ldd);
 							}
 							else
 							{
@@ -845,7 +850,7 @@ gtk_box_pack_start(GTK_BOX(vb),splitw,FALSE,FALSE,3);
 	ui_widget_set_tooltip_text(sdm._##A,_(D));\
 	g_signal_connect(GTK_OBJECT(sdm._##A), "activate", G_CALLBACK(A##_click), NULL); \
 	keybindings_set_item(plugin_key_group, KB_DEBUG_##A, kb_activate, \
-		0, 0, ""#Q2, _(Q2 ":  " D), sdm._##A);
+		0, 0, ""#Q2, Q2 ":  " D, sdm._##A);
 
 #define DBKEYACT(A,B,C) \
 	case KB_DEBUG_##A: \
