@@ -25,6 +25,11 @@
 /* LaTeX plugin */
 /* This plugin improves the work with LaTeX and Geany.*/
 
+
+#ifdef HAVE_CONFIG_H
+	#include "config.h" /* for the gettext domain */
+#endif
+
 #include "geanylatex.h"
 #include "ctype.h"
 
@@ -35,7 +40,7 @@ PLUGIN_SET_TRANSLATABLE_INFO(
 	GETTEXT_PACKAGE,
 	_("GeanyLaTeX"),
 	_("Plugin to provide better LaTeX support"),
-	"0.6pre-1",
+	"0.6",
 	"Frank Lanitz <frank@frank.uvena.de>")
 
 GeanyPlugin	 *geany_plugin;
@@ -85,7 +90,7 @@ static gboolean glatex_autocompletion_active = FALSE;
  * and \endgroup{}. */
 static gint glatex_autocompletion_context_size;
 static gboolean glatex_autocompletion_only_for_latex;
-static gboolean glatex_autobraces_active = TRUE;
+gboolean glatex_autobraces_active = TRUE;
 
 /* Function will be deactivated, when only loaded */
 static gboolean toggle_active = FALSE;
@@ -449,8 +454,6 @@ static void on_document_new(G_GNUC_UNUSED GObject *object, GeanyDocument *doc,
 }
 
 
-
-
 static void
 on_geany_startup_complete(G_GNUC_UNUSED GObject *obj,
 						  G_GNUC_UNUSED gpointer user_data)
@@ -842,7 +845,6 @@ glatex_insert_command_activated(G_GNUC_UNUSED GtkMenuItem * menuitem,
 }
 
 
-
 void
 glatex_insert_ref_activated(G_GNUC_UNUSED GtkMenuItem * menuitem,
 					 G_GNUC_UNUSED gpointer gdata)
@@ -1783,7 +1785,7 @@ glatex_wizard_activated(G_GNUC_UNUSED GtkMenuItem * menuitem,
 	gtk_table_attach_defaults(GTK_TABLE(table), glatex_wizard.encoding_combobox, 1, 2, 2, 3);
 
 	/*  fontsize */
-	label_fontsize = gtk_label_new(_("Font size:"));
+	label_fontsize = gtk_label_new(_("Font size"));
 	glatex_wizard.fontsize_combobox = gtk_combo_box_entry_new_text();
 	gtk_combo_box_append_text(GTK_COMBO_BOX(glatex_wizard.fontsize_combobox),"10pt");
 	gtk_combo_box_append_text(GTK_COMBO_BOX(glatex_wizard.fontsize_combobox),"11pt");
