@@ -1,7 +1,7 @@
 /*
  *      latexstructure.h
  *
- *      Copyright 2009-2010 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
+ *      Copyright 2009-2011 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -37,8 +37,7 @@ enum {
 
 extern const gchar *glatex_structure_values[];
 
-void glatex_structure_lvlup();
-
-void glatex_structure_lvldown();
-
+void glatex_structure_lvlup(void);
+void glatex_structure_lvldown(void);
+gint glatex_structure_rotate(gboolean direction, gint start);
 #endif
