@@ -1,7 +1,7 @@
 /*
- *	letters.h
+ *      letters.h
  *
- *      Copyright 2008-2010 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
+ *      Copyright 2008-2011 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -33,6 +33,6 @@ extern SubMenuTemplate glatex_char_array[];
 
 extern CategoryName glatex_cat_names[];
 
-gchar *glatex_get_entity(const gchar *letter);
+const gchar *glatex_get_entity(const gchar *letter);
 
 #endif
