@@ -39,13 +39,15 @@ GString *glatex_get_template_from_file(gchar *filepath)
 
 static void glatex_init_cutom_template_item(gchar *file, GPtrArray *array)
 {
-	TemplateEntry *template = g_new0(TemplateEntry, 1);
+	TemplateEntry *template = NULL;
 	gchar *tmp = NULL;
 
 	/* Return if its not a searched file */
 	if (g_str_has_suffix(file,".gtl") == FALSE)
 		return;
 
+	template = g_new0(TemplateEntry, 1);
+
 	template->filepath = g_strdup(file);
 
 	tmp = g_path_get_basename(file);
@@ -84,8 +86,6 @@ GPtrArray* glatex_init_custom_templates()
 
 /* Frees all elelements of struct */
 void glatex_free_template_entry(TemplateEntry *template, G_GNUC_UNUSED gpointer *data){
-	if (template->label != NULL)
-		g_free(template->label);
 	if (template->label != NULL)
 		g_free(template->filepath);
 	if (template->label != NULL)
