@@ -1,7 +1,7 @@
 /*
  *      geanylipsum.c
  *
- *      Copyright 2008-2009 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
+ *      Copyright 2008-2011 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -12,27 +12,35 @@
  *      but WITHOUT ANY WARRANTY; without even the implied warranty of
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
- *
  *      You should have received a copy of the GNU General Public License
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  */
 
-#include "geanyplugin.h"
+#ifdef HAVE_CONFIG_H
+	#include "config.h" /* for the gettext domain */
+#endif
 
 #include <string.h>
 #ifdef HAVE_LOCALE_H
 # include <locale.h>
 #endif
 
+#include <geanyplugin.h>
 
 GeanyPlugin		*geany_plugin;
 GeanyData		*geany_data;
 GeanyFunctions	*geany_functions;
 
-PLUGIN_VERSION_CHECK(147)
-PLUGIN_SET_INFO(_("GeanyLipsum"), _("Creating dummy text with Geany"), "0.4.2", "Frank Lanitz <frank@frank.uvena.de>");
+PLUGIN_VERSION_CHECK(188)
+PLUGIN_SET_TRANSLATABLE_INFO(
+	LOCALEDIR,
+	GETTEXT_PACKAGE,
+	_("GeanyLipsum"),
+	_("Creating dummy text with Geany"),
+	"0.4.3",
+	"Frank Lanitz <frank@frank.uvena.de>");
 
 static GtkWidget *main_menu_item = NULL;
 static gchar *lipsum = NULL;
@@ -50,7 +58,7 @@ enum
 	COUNT_KB
 };
 
-PLUGIN_KEY_GROUP(geanylipsum, COUNT_KB);
+
 
 
 
@@ -134,6 +142,7 @@ plugin_init(G_GNUC_UNUSED GeanyData *data)
 	GKeyFile *config = g_key_file_new();
 	GtkTooltips *tooltips = NULL;
 	gchar *config_file = NULL;
+	GeanyKeyGroup *key_group;
 
 	tooltips = gtk_tooltips_new();
 
@@ -165,10 +174,12 @@ plugin_init(G_GNUC_UNUSED GeanyData *data)
 	main_menu_item = menu_lipsum;
 
 	/* init keybindings */
-	keybindings_set_item(plugin_key_group, LIPSUM_KB_INSERT, kblipsum_insert,
+	key_group = plugin_set_key_group(geany_plugin, "geanylipsum", COUNT_KB, NULL);
+	keybindings_set_item(key_group, LIPSUM_KB_INSERT, kblipsum_insert,
 		0, 0, "insert_lipsum", _("Insert Lipsum text"), menu_lipsum);
 }
 
+
 /* Called by Geany before unloading the plugin. */
 void plugin_cleanup(void)
 {
@@ -178,4 +189,3 @@ void plugin_cleanup(void)
 	/* free lipsum snippet */
 	g_free(lipsum);
 }
-
