@@ -20,11 +20,14 @@
 
 #include <string.h>
 
-#include "geanyplugin.h"
+#include <geanyplugin.h>
 #include "geanyvc.h"
 
 extern GeanyFunctions *geany_functions;
 
+const gchar *get_external_diff_viewer(void);
+
+
 enum
 {
 	EXTERNAL_DIFF_MELD,
@@ -35,7 +38,8 @@ enum
 	EXTERNAL_DIFF_COUNT
 };
 
-static gchar *viewers[EXTERNAL_DIFF_COUNT] = { "meld", "kompare", "kdiff3", "diffuse", "tkdiff" };
+
+static const gchar *viewers[EXTERNAL_DIFF_COUNT] = { "meld", "kompare", "kdiff3", "diffuse", "tkdiff" };
 
 static gchar *extern_diff_viewer = NULL;
 const gchar *
@@ -50,13 +54,14 @@ get_external_diff_viewer()
 	{
 		if (g_find_program_in_path(viewers[i]))
 		{
-			extern_diff_viewer = viewers[i];
-			return extern_diff_viewer;
+			extern_diff_viewer = (gchar *) viewers[i];
+			return viewers[i];
 		}
 	}
 	return NULL;
 }
 
+
 void
 vc_external_diff(const gchar * src, const gchar * dest)
 {
