@@ -29,11 +29,13 @@
 #include <glib/gstdio.h>
 #include <unistd.h>
 
-#include "geanyvc.h"
-#include "geanyplugin.h"
-#include "SciLexer.h"
+#ifdef HAVE_CONFIG_H
+	#include "config.h"
+#endif
+#include <geanyplugin.h>
 
-#include <config.h>
+#include "geanyvc.h"
+#include "SciLexer.h"
 
 #ifdef USE_GTKSPELL
 #include <gtkspell/gtkspell.h>
@@ -51,7 +53,7 @@ PLUGIN_SET_TRANSLATABLE_INFO(
 	_("GeanyVC"),
 	_("Interface to different Version Control systems."),
 	VERSION,
-	_("Yura Siamashka <yurand2@gmail.com>,\nFrank Lanitz <frank@frank.uvena.de>"));
+	"Yura Siamashka <yurand2@gmail.com>,\nFrank Lanitz <frank@frank.uvena.de>");
 
 /* Some global variables */
 static gboolean set_changed_flag;
@@ -466,8 +468,8 @@ execute_custom_command(const gchar * dir, const gchar ** argv, const gchar ** en
 			g_error_free(error);
 		}
 
-		// need to convert output text from the encoding of the original file into
-		// UTF-8 because internally Geany always needs UTF-8
+		/* need to convert output text from the encoding of the original file into
+		   UTF-8 because internally Geany always needs UTF-8 */
 		if (std_out && *std_out)
 		{
 			tmp = g_string_new(*std_out);
@@ -514,7 +516,7 @@ static gint
 execute_command(const VC_RECORD * vc, gchar ** std_out, gchar ** std_err, const gchar * filename,
 		gint cmd, GSList * list, const gchar * message)
 {
-	gchar *dir;
+	gchar *dir = NULL;
 	gint ret;
 
 	if (std_out)
@@ -651,7 +653,7 @@ vcdiff_dir_activated(G_GNUC_UNUSED GtkMenuItem * menuitem, gpointer data)
 	doc = document_get_current();
 	g_return_if_fail(doc != NULL && doc->file_name != NULL);
 
-	if (doc && doc->changed && doc->file_name != NULL)
+	if (doc->changed)
 	{
 		document_save_file(doc, FALSE);
 	}
@@ -708,7 +710,7 @@ vcblame_activated(G_GNUC_UNUSED GtkMenuItem * menuitem, G_GNUC_UNUSED gpointer g
 	}
 	else
 	{
-		ui_set_statusbar(FALSE, _("No history avaible"));
+		ui_set_statusbar(FALSE, _("No history available"));
 	}
 }
 
@@ -1411,7 +1413,7 @@ create_commitDialog(void)
 	gtk_widget_show(btnCancel);
 	gtk_dialog_add_action_widget(GTK_DIALOG(commitDialog), btnCancel, GTK_RESPONSE_CANCEL);
 
-	btnCommit = gtk_button_new_with_mnemonic(_("_Commit"));
+	btnCommit = gtk_button_new_with_mnemonic(_("C_ommit"));
 	gtk_widget_show(btnCommit);
 	gtk_dialog_add_action_widget(GTK_DIALOG(commitDialog), btnCommit, GTK_RESPONSE_APPLY);
 
@@ -1783,7 +1785,7 @@ on_configure_response(G_GNUC_UNUSED GtkDialog * dialog, gint response,
 		}
 		else
 		{
-			// write config to file
+			/* write config to file */
 			data = g_key_file_to_data(config, NULL, NULL);
 			utils_write_file(config_file, data);
 			g_free(data);
@@ -1942,7 +1944,7 @@ load_config()
 	lang = g_key_file_get_string(config, "VC", "spellchecking_language", &error);
 	if (error != NULL)
 	{
-		// Set default value. Using system standard language.
+		/* Set default value. Using system standard language. */
 		lang = NULL;
 		g_error_free(error);
 		error = NULL;
@@ -2024,7 +2026,7 @@ do_current_file_menu(GtkWidget ** parent_menu, gboolean editor_menu)
 	menu_vc_show_file = gtk_menu_item_new_with_mnemonic(_("_Original"));
 	gtk_container_add(GTK_CONTAINER(cur_file_menu), menu_vc_show_file);
 	ui_widget_set_tooltip_text(menu_vc_show_file,
-			     _("Shows the orignal of the current file"));
+			     _("Shows the original of the current file"));
 
 	g_signal_connect(menu_vc_show_file, "activate", G_CALLBACK(vcshow_file_activated), NULL);
 
@@ -2214,8 +2216,6 @@ plugin_init(G_GNUC_UNUSED GeanyData * data)
 	GtkWidget *menu_vc_dir = NULL;
 	GtkWidget *menu_vc_basedir = NULL;
 
-	main_locale_init(LOCALEDIR, GETTEXT_PACKAGE);
-
 	config_file =
 		g_strconcat(geany->app->configdir, G_DIR_SEPARATOR_S, "plugins", G_DIR_SEPARATOR_S,
 			    "VC", G_DIR_SEPARATOR_S, "VC.conf", NULL);
