@@ -1,5 +1,5 @@
 /*
- *      Copyright 2007-2009 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
+ *      Copyright 2007-2011 Frank Lanitz <frank(at)frank(dot)uvena(dot)de>
  *      Copyright 2007-2009 Enrico Tröger <enrico.troeger@uvena.de>
  *      Copyright 2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *      Copyright 2007-2009 Yura Siamashka <yurand2@gmail.com>
@@ -20,7 +20,7 @@
  */
 
 #include <string.h>
-#include "geanyplugin.h"
+#include <geanyplugin.h>
 #include "geanyvc.h"
 
 extern GeanyData *geany_data;
@@ -119,7 +119,7 @@ static gboolean
 in_vc_bzr(const gchar * filename)
 {
 	gint exit_code;
-	gchar *argv[] = { "bzr", "log", NULL, NULL };
+	const gchar *argv[] = { "bzr", "log", NULL, NULL };
 	gchar *dir;
 	gchar *base_name;
 	gboolean ret = FALSE;
@@ -163,7 +163,7 @@ get_commit_files_bzr(const gchar * dir)
 		FILE_NAME,
 	};
 
-	gchar *txt;
+	gchar *txt = NULL;
 	GSList *ret = NULL;
 	gint pstatus = FIRST_CHAR;
 	const gchar *p;
@@ -172,7 +172,7 @@ get_commit_files_bzr(const gchar * dir)
 	const gchar *start = NULL;
 	CommitItem *item;
 
-	const gchar *status;
+	const gchar *status = NULL;
 	gchar *filename;
 	const char *argv[] = { "bzr", "status", "--short", NULL };
 
@@ -198,14 +198,8 @@ get_commit_files_bzr(const gchar * dir)
 				status = FILE_STATUS_ADDED;
 			else if (*p == '-')
 				status = FILE_STATUS_DELETED;
-			// rename
-			//else if (*p == 'R')
 			else if (*p == '?')
 				status = FILE_STATUS_UNKNOWN;
-			// conflicts
-			//else if (*p == 'C')
-			// pending merge
-			//else if (*p == 'P')
 			pstatus = SECOND_CHAR;
 		}
 		else if (pstatus == SECOND_CHAR)
@@ -214,16 +208,12 @@ get_commit_files_bzr(const gchar * dir)
 				status = FILE_STATUS_ADDED;
 			else if (*p == 'D')
 				status = FILE_STATUS_DELETED;
-			// file kind changed
-			//else if (*p == 'K')
 			else if (*p == 'M')
 				status = FILE_STATUS_MODIFIED;
 			pstatus = THIRD_CHAR;
 		}
 		else if (pstatus == THIRD_CHAR)
 		{
-			// execute bit change
-			//if (*p == '*')
 			pstatus = SKIP_SPACE;
 		}
 		else if (pstatus == SKIP_SPACE)
