@@ -3,8 +3,8 @@
  *      Copyright 2007-2009 Enrico Tröger <enrico.troeger@uvena.de>
  *      Copyright 2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *      Copyright 2007-2009 Yura Siamashka <yurand2@gmail.com>
- *      Copyright 2010 Jean-Paul Iribarren <jeanpaul.Iribarren@free.fr> 
- * 
+ *      Copyright 2010 Jean-Paul Iribarren <jeanpaul.Iribarren@free.fr>
+ *
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <string.h>
-#include "geanyplugin.h"
+#include <geanyplugin.h>
 #include "geanyvc.h"
 
 static const gchar *CVS_CMD_DIFF_FILE[] = { "cvs", "diff", "-u", BASE_FILENAME, NULL };
@@ -162,7 +162,7 @@ get_commit_files_cvs(const gchar * dir)
 	const gchar *start = NULL;
 	CommitItem *item;
 
-	const gchar *status;
+	const gchar *status = NULL;
 	gchar *filename;
 	const char *argv[] = { "cvs", "-nq", "update", NULL };
 
@@ -190,7 +190,7 @@ get_commit_files_cvs(const gchar * dir)
 
 			if (!status || *(p + 1) != ' ')
 			{
-				// skip unknown status line
+				/* skip unknown status line */
 				while (*p)
 				{
 					p++;
