@@ -20,7 +20,7 @@
  */
 
 #include <string.h>
-#include "geanyplugin.h"
+#include <geanyplugin.h>
 #include "geanyvc.h"
 
 extern GeanyData *geany_data;
@@ -118,7 +118,7 @@ static gboolean
 in_vc_hg(const gchar * filename)
 {
 	gint exit_code;
-	gchar *argv[] = { "hg", "status", "-mac", NULL, NULL };
+	const gchar *argv[] = { "hg", "status", "-mac", NULL, NULL };
 	gchar *dir;
 	gchar *base_name;
 	gboolean ret = FALSE;
@@ -167,7 +167,7 @@ get_commit_files_hg(const gchar * dir)
 	const gchar *start = NULL;
 	CommitItem *item;
 
-	const gchar *status;
+	const gchar *status = NULL;
 	gchar *filename;
 	const char *argv[] = { "hg", "status", NULL };
 
