@@ -22,6 +22,12 @@
 //========================================== INCLUDES =========================================================
 
 #include <gtk/gtk.h>
+
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+#include <geanyplugin.h>
+
 #include "PrettyPrinter.h"
 
 //========================================== VARIABLES ========================================================
@@ -31,6 +37,6 @@ extern PrettyPrintingOptions* prettyPrintingOptions;
 //========================================== FUNCTIONS ========================================================
 
 GtkWidget* createPrettyPrinterConfigUI(GtkDialog* dialog);
-void saveSettings();
+void saveSettings(void);
 
 #endif
