@@ -19,14 +19,16 @@
 #ifndef PLUGIN_ENTRY_H
 #define PLUGIN_ENTRY_H
 
-// This constants will be enabled once the preferences GUI will be finished :)
-//#define PLUGIN_XMLPP_GUI
-
 //========================================== INCLUDES ==========================================================
 
 #include <stdlib.h>
 #include <stdio.h>
+
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 #include <geanyplugin.h>
+
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 #include "PrettyPrinter.h"
