@@ -20,36 +20,41 @@
 
 //======================= FUNCTIONS ====================================================================
 
+//error reporting functions
+static void PP_ERROR(const char* fmt, ...) G_GNUC_PRINTF(1,2);  //prints an error message
+
 //xml pretty printing functions
 static void putCharInBuffer(char charToAdd);                     //put a char into the new char buffer
-static void putCharsInBuffer(char* charsToAdd);                  //put the chars into the new char buffer
+static void putCharsInBuffer(const char* charsToAdd);            //put the chars into the new char buffer
 static void putNextCharsInBuffer(int nbChars);                   //put the next nbChars of the input buffer into the new buffer
-static int readWhites();                                         //read the next whites into the input buffer
-static char readNextChar();                                      //read the next char into the input buffer;
-static char getNextChar();                                       //returns the next char but do not increase the input buffer index (use readNextChar for that)
-static char getPreviousInsertedChar();                           //returns the last inserted char into the new buffer
-static gboolean isWhite(char c);                                 //check if the specified char is a white
-static gboolean isLineBreak(char c);                             //check if the specified char is a new line
-static gboolean isQuote(char c);                                 //check if the specified char is a quote (simple or double)
-static int putNewLine();                                         //put a new line into the new char buffer with the correct number of whites (indentation)
-static gboolean isInlineNodeAllowed();                           //check if it is possible to have an inline node
-static void resetBackwardIndentation(gboolean resetLineBreak);   //reset the indentation for the current depth (just reset the index in fact)
+static int readWhites(bool considerLineBreakAsWhite);            //read the next whites into the input buffer
+static char readNextChar(void);                                  //read the next char into the input buffer;
+static char getNextChar(void);                                   //returns the next char but do not increase the input buffer index (use readNextChar for that)
+static char getPreviousInsertedChar(void);                       //returns the last inserted char into the new buffer
+static bool isWhite(char c);                                     //check if the specified char is a white
+static bool isSpace(char c);                                     //check if the specified char is a space
+static bool isLineBreak(char c);                                 //check if the specified char is a new line
+static bool isQuote(char c);                                     //check if the specified char is a quote (simple or double)
+static int putNewLine(void);                                     //put a new line into the new char buffer with the correct number of whites (indentation)
+static bool isInlineNodeAllowed(void);                           //check if it is possible to have an inline node
+static bool isOnSingleLine(int skip, char stop1, char stop2);    //check if the current node data is on one line (for inlining)
+static void resetBackwardIndentation(bool resetLineBreak);       //reset the indentation for the current depth (just reset the index in fact)
                                                              
 //specific parsing functions                                 
-static int processElements();                                    //returns the number of elements processed
-static void processElementAttribute();                           //process on attribute of a node
-static void processElementAttributes();                          //process all the attributes of a node
-static void processHeader();                                     //process the header <?xml version="..." ?>
-static void processNode();                                       //process an XML node
-static void processTextNode();                                   //process a text node
-static void processComment();                                    //process a comment
-static void processCDATA();                                      //process a CDATA node
-static void processDoctype();                                    //process a DOCTYPE node
-static void processDoctypeElement();                             //process a DOCTYPE ELEMENT node
-                                                             
+static int processElements(void);                                //returns the number of elements processed
+static void processElementAttribute(void);                       //process on attribute of a node
+static void processElementAttributes(void);                      //process all the attributes of a node
+static void processHeader(void);                                 //process the header <?xml version="..." ?>
+static void processNode(void);                                   //process an XML node
+static void processTextNode(void);                               //process a text node
+static void processComment(void);                                //process a comment
+static void processCDATA(void);                                  //process a CDATA node
+static void processDoctype(void);                                //process a DOCTYPE node
+static void processDoctypeElement(void);                         //process a DOCTYPE ELEMENT node
+
 //debug function                                             
-static void printError(char *msg, ...);                          //just print a message like the printf method
-static void printDebugStatus();                                  //just print some variables into the console for debugging
+static void printError(const char *msg, ...) G_GNUC_PRINTF(1,2); //just print a message like the printf method
+static void printDebugStatus(void);                              //just print some variables into the console for debugging
 
 //============================================ PRIVATE PROPERTIES ======================================
 
@@ -65,12 +70,22 @@ static int inputBufferLength;                                     //input buffer
 static int inputBufferIndex;                                      //input buffer index (position of the next char to read into the input string)
 static int currentDepth;                                          //current depth (for indentation)
 static char* currentNodeName;                                     //current node name
-static gboolean appendIndentation;                                //if the indentation must be added (with a line break before)
-static gboolean lastNodeOpen;                                     //defines if the last action was a not opening or not
+static bool appendIndentation;                                //if the indentation must be added (with a line break before)
+static bool lastNodeOpen;                                     //defines if the last action was a not opening or not
 static PrettyPrintingOptions* options;                            //options of PrettyPrinting
 
 //============================================ GENERAL FUNCTIONS =======================================
 
+static void PP_ERROR(const char* fmt, ...)
+{
+    va_list va;
+    
+    va_start(va, fmt);
+    vfprintf(stderr, fmt, va);
+    putc('\n', stderr);
+    va_end(va);
+}
+
 int processXMLPrettyPrinting(char** buffer, int* length, PrettyPrintingOptions* ppOptions)
 {
     //empty buffer, nothing to process
@@ -79,10 +94,14 @@ int processXMLPrettyPrinting(char** buffer, int* length, PrettyPrintingOptions*
     
     //initialize the variables
     result = PRETTY_PRINTING_SUCCESS;
-    gboolean freeOptions = FALSE;
-    if (ppOptions == NULL) { ppOptions = createDefaultPrettyPrintingOptions(); freeOptions = TRUE; }
-    options = ppOptions;
+    bool freeOptions = FALSE;
+    if (ppOptions == NULL) 
+    { 
+        ppOptions = createDefaultPrettyPrintingOptions(); 
+        freeOptions = TRUE; 
+    }
     
+    options = ppOptions;
     currentNodeName = NULL;
     appendIndentation = FALSE;
     lastNodeOpen = FALSE;
@@ -95,10 +114,10 @@ int processXMLPrettyPrinting(char** buffer, int* length, PrettyPrintingOptions*
     
     xmlPrettyPrintedLength = *length;
     xmlPrettyPrinted = (char*)malloc(sizeof(char)*(*length));
-    if (xmlPrettyPrinted == NULL) { g_error("Allocation error"); }
+    if (xmlPrettyPrinted == NULL) { PP_ERROR("Allocation error (initialisation)"); return PRETTY_PRINTING_SYSTEM_ERROR; }
     
     //go to the first char
-    readWhites();
+    readWhites(TRUE);
 
     //process the pretty-printing
     processElements();
@@ -107,8 +126,9 @@ int processXMLPrettyPrinting(char** buffer, int* length, PrettyPrintingOptions*
     putCharInBuffer('\0');
     
     //adjust the final size
-    xmlPrettyPrinted = realloc(xmlPrettyPrinted, xmlPrettyPrintedIndex);
-    if (xmlPrettyPrinted == NULL) { g_error("Allocation error"); }
+    char* reallocated = (char*)realloc(xmlPrettyPrinted, xmlPrettyPrintedIndex); 
+    if (reallocated == NULL) { PP_ERROR("Allocation error (reallocation size is %d)", xmlPrettyPrintedIndex); return PRETTY_PRINTING_SYSTEM_ERROR; }
+    xmlPrettyPrinted = reallocated;
     
     //freeing the unused values
     if (freeOptions) { free(options); }
@@ -136,31 +156,34 @@ int processXMLPrettyPrinting(char** buffer, int* length, PrettyPrintingOptions*
     return result;
 }
 
-PrettyPrintingOptions* createDefaultPrettyPrintingOptions()
+PrettyPrintingOptions* createDefaultPrettyPrintingOptions(void)
 {
-    PrettyPrintingOptions* options = (PrettyPrintingOptions*)malloc(sizeof(PrettyPrintingOptions));
-    if (options == NULL) 
+    PrettyPrintingOptions* defaultOptions = (PrettyPrintingOptions*)malloc(sizeof(PrettyPrintingOptions));
+    if (defaultOptions == NULL) 
     { 
-        g_error("Unable to allocate memory for PrettyPrintingOptions");
-        return NULL; 
+        PP_ERROR("Unable to allocate memory for PrettyPrintingOptions");
+        return NULL;
     }
     
-    options->newLineChars = "\r\n";
-    options->indentChar = ' ';
-    options->indentLength = 2;
-    options->oneLineText = TRUE;
-    options->inlineText = TRUE;
-    options->oneLineComment = TRUE;
-    options->inlineComment = TRUE;
-    options->oneLineCdata = TRUE;
-    options->inlineCdata = TRUE;
-    options->emptyNodeStripping = TRUE;
-    options->emptyNodeStrippingSpace = TRUE;
-    options->forceEmptyNodeSplit = FALSE;
-    options->trimLeadingWhites = TRUE;
-    options->trimTrailingWhites = TRUE;
+    defaultOptions->newLineChars = "\r\n";
+    defaultOptions->indentChar = ' ';
+    defaultOptions->indentLength = 2;
+    defaultOptions->oneLineText = FALSE;
+    defaultOptions->inlineText = TRUE;
+    defaultOptions->oneLineComment = FALSE;
+    defaultOptions->inlineComment = TRUE;
+    defaultOptions->oneLineCdata = FALSE;
+    defaultOptions->inlineCdata = TRUE;
+    defaultOptions->emptyNodeStripping = TRUE;
+    defaultOptions->emptyNodeStrippingSpace = TRUE;
+    defaultOptions->forceEmptyNodeSplit = FALSE;
+    defaultOptions->trimLeadingWhites = TRUE;
+    defaultOptions->trimTrailingWhites = TRUE;
+    defaultOptions->alignComment = TRUE;
+    defaultOptions->alignText = TRUE;
+    defaultOptions->alignCdata = TRUE;
     
-    return options;
+    return defaultOptions;
 }
 
 void putNextCharsInBuffer(int nbChars)
@@ -180,8 +203,9 @@ void putCharInBuffer(char charToAdd)
     {
         if (charToAdd == '\0') { ++xmlPrettyPrintedLength; }
         else { xmlPrettyPrintedLength += inputBufferLength; }
-        xmlPrettyPrinted = (char*)realloc(xmlPrettyPrinted, xmlPrettyPrintedLength);
-        if (xmlPrettyPrinted == NULL) { g_error("Allocation error"); }
+        char* reallocated = (char*)realloc(xmlPrettyPrinted, xmlPrettyPrintedLength);
+        if (reallocated == NULL) { PP_ERROR("Allocation error (char was %c)", charToAdd); return; }
+        xmlPrettyPrinted = reallocated;
     }
     
     //putting the char and increase the index for the next one
@@ -189,22 +213,22 @@ void putCharInBuffer(char charToAdd)
     ++xmlPrettyPrintedIndex;
 }
 
-void putCharsInBuffer(char* charsToAdd)
+void putCharsInBuffer(const char* charsToAdd)
 {
-    int index = 0;
-    while (charsToAdd[index] != '\0')
+    int currentIndex = 0;
+    while (charsToAdd[currentIndex] != '\0')
     {
-        putCharInBuffer(charsToAdd[index]);
-        ++index;
+        putCharInBuffer(charsToAdd[currentIndex]);
+        ++currentIndex;
     }
 }
 
-char getPreviousInsertedChar()
+char getPreviousInsertedChar(void)
 {
     return xmlPrettyPrinted[xmlPrettyPrintedIndex-1];
 }
 
-int putNewLine()
+int putNewLine(void)
 {
     putCharsInBuffer(options->newLineChars);
     int spaces = currentDepth*options->indentLength;
@@ -217,20 +241,22 @@ int putNewLine()
     return spaces;
 }
 
-char getNextChar()
+char getNextChar(void)
 {
     return inputBuffer[inputBufferIndex];
 }
 
-char readNextChar()
+char readNextChar(void)
 {   
     return inputBuffer[inputBufferIndex++];
 }
 
-int readWhites()
+int readWhites(bool considerLineBreakAsWhite)
 {
     int counter = 0;
-    while(isWhite(inputBuffer[inputBufferIndex]))
+    while(isWhite(inputBuffer[inputBufferIndex]) && 
+          (!isLineBreak(inputBuffer[inputBufferIndex]) || 
+           considerLineBreakAsWhite))
     {
         ++counter;
         ++inputBufferIndex;
@@ -239,33 +265,31 @@ int readWhites()
     return counter;
 }
 
-gboolean isQuote(char c)
+bool isQuote(char c)
 {
-    if (c == '\'') return TRUE;
-    if (c == '"') return TRUE;
-    
-    return FALSE;
+    return (c == '\'' ||
+            c == '\"');
 }
 
-gboolean isWhite(char c)
+bool isWhite(char c)
 {
-    if (c == ' ') return TRUE;
-    if (c == '\t') return TRUE;
-    if (c == '\r') return TRUE;
-    if (c == '\n') return TRUE;
-
-    return FALSE;
+    return (isSpace(c) ||
+            isLineBreak(c));
 }
 
-gboolean isLineBreak(char c)
+bool isSpace(char c)
 {
-    if (c == '\n') return TRUE;
-    if (c == '\r') return TRUE;
-    
-    return FALSE;
+    return (c == ' ' ||
+            c == '\t');
 }
 
-gboolean isInlineNodeAllowed()
+bool isLineBreak(char c)
+{
+    return (c == '\n' || 
+            c == '\r');
+}
+
+bool isInlineNodeAllowed(void)
 {
     //the last action was not an opening => inline not allowed
     if (!lastNodeOpen) { return FALSE; }
@@ -274,13 +298,14 @@ gboolean isInlineNodeAllowed()
     int secondChar = inputBuffer[inputBufferIndex+1]; //should be '!'
     int thirdChar = inputBuffer[inputBufferIndex+2]; //should be '-' or '['
     
-    int index = inputBufferIndex+1;
+    //loop through the content up to the next opening/closing node
+    int currentIndex = inputBufferIndex+1;
     if (firstChar == '<')
     {
         //another node is being open ==> no inline !
         if (secondChar != '!') { return FALSE; }
         
-        //okay we are in a comment node, so read until it is closed
+        //okay we are in a comment/cdata node, so read until it is closed
         
         //select the closing char
         char closingComment = '-';
@@ -288,37 +313,37 @@ gboolean isInlineNodeAllowed()
         
         //read until closing
         char oldChar = ' ';
-        index += 3; //that by pass meanless chars
-        gboolean loop = TRUE;
+        currentIndex += 3; //that bypass meanless chars
+        bool loop = TRUE;
         while (loop)
         {
-            char current = inputBuffer[index];
-            if (current == closingComment && oldChar == closingComment) { loop = FALSE; } //end of comment
+            char current = inputBuffer[currentIndex];
+            if (current == closingComment && oldChar == closingComment) { loop = FALSE; } //end of comment/cdata
             oldChar = current;
-            ++index;
+            ++currentIndex;
         }
         
         //okay now avoid blanks
         // inputBuffer[index] is now '>'
-        ++index;
-        while (isWhite(inputBuffer[index])) { ++index; }
+        ++currentIndex;
+        while (isWhite(inputBuffer[currentIndex])) { ++currentIndex; }
     }
     else
     {
         //this is a text node. Simply loop to the next '<'
-        while (inputBuffer[index] != '<') { ++index; }
+        while (inputBuffer[currentIndex] != '<') { ++currentIndex; }
     }
     
     //check what do we have now
-    char currentChar = inputBuffer[index];
+    char currentChar = inputBuffer[currentIndex];
     if (currentChar == '<')
     {
         //check if that is a closing node
-        currentChar = inputBuffer[index+1];
+        currentChar = inputBuffer[currentIndex+1];
         if (currentChar == '/')
         {
             //as we are in a correct XML (so far...), if the node is 
-            //being directly close, the inline is allowed !!!
+            //being directly closed, the inline is allowed !!!
             return TRUE;
         }
     }
@@ -327,7 +352,49 @@ gboolean isInlineNodeAllowed()
     return FALSE;
 }
 
-void resetBackwardIndentation(gboolean resetLineBreak)
+bool isOnSingleLine(int skip, char stop1, char stop2)
+{
+    int currentIndex = inputBufferIndex+skip; //skip the n first chars (in comment <!--)
+    bool onSingleLine = TRUE;
+    
+    char oldChar = inputBuffer[currentIndex];
+    char currentChar = inputBuffer[currentIndex+1];
+    while(onSingleLine && oldChar != stop1 && currentChar != stop2)
+    {
+        onSingleLine = !isLineBreak(oldChar);
+        
+        ++currentIndex;
+        oldChar = currentChar;
+        currentChar = inputBuffer[currentIndex+1];
+        
+        /**
+         * A line break inside the node has been reached. But we should check
+         * if there is something before the end of the node (otherwise, there
+         * are only spaces and it may be wanted to be considered as a single
+         * line). //TODO externalize an option for that ?
+         */
+        if (!onSingleLine)
+        {
+            while(oldChar != stop1 && currentChar != stop2)
+            {
+                //okay there is something else => this is not on one line
+                if (!isWhite(oldChar)) return FALSE;
+              
+                ++currentIndex;
+                oldChar = currentChar;
+                currentChar = inputBuffer[currentIndex+1];
+            }
+            
+            //the end of the node has been reached with only whites. Then
+            //the node can be considered being one single line
+            return TRUE;
+        }
+    }
+    
+    return onSingleLine;
+}
+
+void resetBackwardIndentation(bool resetLineBreak)
 {
     xmlPrettyPrintedIndex -= (currentDepth*options->indentLength);
     if (resetLineBreak) 
@@ -347,15 +414,15 @@ void resetBackwardIndentation(gboolean resetLineBreak)
 //-----------------------------------------------------------------------------------------------------------------------------------------
 //#########################################################################################################################################
 
-int processElements()
+int processElements(void)
 {
     int counter = 0;
     ++currentDepth;
-    gboolean loop = TRUE;
+    bool loop = TRUE;
     while (loop && result == PRETTY_PRINTING_SUCCESS)
     {
         //strip unused whites
-        readWhites();
+        readWhites(TRUE);
         
         char nextChar = getNextChar();
         if (nextChar == '\0') { return 0; } //no more data to read
@@ -364,7 +431,7 @@ int processElements()
         if (appendIndentation) { putNewLine(); }
         
         //always append indentation (but need to store the state)
-        gboolean indentBackward = appendIndentation;
+        bool indentBackward = appendIndentation;
         appendIndentation = TRUE; 
         
         //okay what do we have now ?
@@ -418,7 +485,7 @@ int processElements()
     return counter;
 }
 
-void processElementAttribute()
+void processElementAttribute(void)
 {
     //process the attribute name
     char nextChar = readNextChar();
@@ -446,7 +513,7 @@ void processElementAttribute()
     putCharInBuffer(quote);
 }
 
-void processElementAttributes()
+void processElementAttributes(void)
 {
     char current = getNextChar(); //should not be a white
     if (isWhite(current)) 
@@ -456,10 +523,10 @@ void processElementAttributes()
         return; 
     }
     
-    gboolean loop = TRUE;
+    bool loop = TRUE;
     while (loop)
     {
-        readWhites(); //strip the whites
+        readWhites(TRUE); //strip the whites
         
         char next = getNextChar(); //don't read the last char (processed afterwards)
         if (next == '/') { loop = FALSE; } /* end of node */
@@ -473,7 +540,7 @@ void processElementAttributes()
     }
 }
 
-void processHeader()
+void processHeader(void)
 {
     int firstChar = inputBuffer[inputBufferIndex]; //should be '<'
     int secondChar = inputBuffer[inputBufferIndex+1]; //must be '?'
@@ -492,7 +559,7 @@ void processHeader()
         
         while(!isWhite(getNextChar())) { putNextCharsInBuffer(1); }
         
-        readWhites();
+        readWhites(TRUE);
         processElementAttributes(); 
         
         //puts the '?' and '>' chars into the new buffer
@@ -500,7 +567,7 @@ void processHeader()
     }
 }
 
-void processNode()
+void processNode(void)
 {
     int opening = readNextChar();
     if (opening != '<') 
@@ -524,20 +591,20 @@ void processNode()
 
     //store the name
     char* nodeName = (char*)malloc(sizeof(char)*nodeNameLength+1);
-    if (nodeName == NULL) { g_error("Allocation error"); }
+    if (nodeName == NULL) { PP_ERROR("Allocation error (node name length is %d)", nodeNameLength); return ; }
     nodeName[nodeNameLength] = '\0';
     int i;
     for (i=0 ; i<nodeNameLength ; ++i)
     {
-        int index = xmlPrettyPrintedIndex-nodeNameLength+i;
-        nodeName[i] = xmlPrettyPrinted[index];
+        int tempIndex = xmlPrettyPrintedIndex-nodeNameLength+i;
+        nodeName[i] = xmlPrettyPrinted[tempIndex];
     }
     
     currentNodeName = nodeName; //set the name for using in other methods
     lastNodeOpen = TRUE;
 
     //process the attributes    
-    readWhites();
+    readWhites(TRUE);
     processElementAttributes();
     
     //process the end of the tag
@@ -576,7 +643,7 @@ void processNode()
     { 
         //the tag is just closed (maybe some content)
         putNextCharsInBuffer(1); 
-        subElementsProcessed = processElements(TRUE); 
+        subElementsProcessed = processElements();
     } 
     else 
     { 
@@ -638,16 +705,17 @@ void processNode()
     currentNodeName = NULL;
 }
 
-void processComment()
+void processComment(void)
 {
-    gboolean inlineAllowed = FALSE;
+    bool inlineAllowed = FALSE;
     if (options->inlineComment) { inlineAllowed = isInlineNodeAllowed(); }
+    if (inlineAllowed && !options->oneLineComment) { inlineAllowed = isOnSingleLine(4, '-', '-'); }
     if (inlineAllowed) { resetBackwardIndentation(TRUE); }
     
     putNextCharsInBuffer(4); //add the chars '<!--'
     
     char oldChar = '-';
-    gboolean loop = TRUE;
+    bool loop = TRUE;
     while (loop)
     {
         char nextChar = readNextChar();
@@ -658,23 +726,71 @@ void processComment()
         
         if (!isLineBreak(nextChar)) //the comment simply continues
         {
-            putCharInBuffer(nextChar);
-            oldChar = nextChar;
-        }
-        else if (!options->oneLineComment) //oh ! there is a line break
-        {
-            readWhites(); //strip the whites and new line
-            putNewLine(); //put a new indentation line
-            oldChar = ' '; //and update the last char
+            if (options->oneLineComment && isSpace(nextChar))
+            {
+                //removes all the unecessary spaces
+                while(isSpace(getNextChar()))
+                {
+                    nextChar = readNextChar();
+                }
+                putCharInBuffer(' ');
+                oldChar = ' ';
+            }
+            else
+            {
+                //comment is left untouched
+                putCharInBuffer(nextChar);
+                oldChar = nextChar;
+            }
             
-            //TODO manage relative spacing
+            if (!loop && options->alignComment) //end of comment
+            {
+                //ensures the chars preceding the first '-' are all spaces (there are at least
+                //5 spaces in front of the '-->' for the alignment with '<!--')
+                bool onlySpaces = xmlPrettyPrinted[xmlPrettyPrintedIndex-3] == ' ' &&
+                                  xmlPrettyPrinted[xmlPrettyPrintedIndex-4] == ' ' &&
+                                  xmlPrettyPrinted[xmlPrettyPrintedIndex-5] == ' ' &&
+                                  xmlPrettyPrinted[xmlPrettyPrintedIndex-6] == ' ' &&
+                                  xmlPrettyPrinted[xmlPrettyPrintedIndex-7] == ' ';
+                
+                //if all the preceding chars are white, then go for replacement
+                if (onlySpaces)
+                {
+                    xmlPrettyPrintedIndex -= 7; //remove indentation spaces
+                    putCharsInBuffer("--"); //reset the first chars of '-->'
+                }
+            }
+        }
+        else if (!options->oneLineComment && !inlineAllowed) //oh ! there is a line break
+        {
+            //if the comments need to be aligned, just add 5 spaces
+            if (options->alignComment) 
+            {
+                int read = readWhites(FALSE); //strip the whites and new line
+                if (nextChar == '\r' && read == 0 && getNextChar() == '\n') //handles the \r\n return line
+                {
+                    readNextChar(); 
+                    readWhites(FALSE);
+                }
+              
+                putNewLine(); //put a new indentation line
+                putCharsInBuffer("     "); //align with <!-- 
+                oldChar = ' '; //and update the last char
+            }
+            else
+            {
+                putCharInBuffer(nextChar);
+                oldChar = nextChar;
+            }
         }
         else //the comments must be inlined
         {
-            readWhites(); //strip the whites and add a space if needed
-            if (getPreviousInsertedChar() != ' ') 
+            readWhites(TRUE); //strip the whites and add a space if needed
+            if (getPreviousInsertedChar() != ' ' &&
+                strncmp(xmlPrettyPrinted+xmlPrettyPrintedIndex-4, "<!--", 4) != 0) //prevents adding a space at the beginning 
             { 
                 putCharInBuffer(' '); 
+                oldChar = ' ';
             }
         }
     }
@@ -694,19 +810,23 @@ void processComment()
     lastNodeOpen = FALSE;
 }
 
-void processTextNode()
+void processTextNode(void)
 {
     //checks if inline is allowed
-    gboolean inlineTextAllowed = FALSE;
+    bool inlineTextAllowed = FALSE;
     if (options->inlineText) { inlineTextAllowed = isInlineNodeAllowed(); }
-    if (inlineTextAllowed) { resetBackwardIndentation(TRUE); } //remove previous indentation
-    
+    if (inlineTextAllowed && !options->oneLineText) { inlineTextAllowed = isOnSingleLine(0, '<', '/'); }
+    if (inlineTextAllowed || !options->alignText) 
+    { 
+        resetBackwardIndentation(TRUE); //remove previous indentation
+        if (!inlineTextAllowed) { putNewLine(); }
+    } 
+   
     //the leading whites are automatically stripped. So we re-add it
     if (!options->trimLeadingWhites)
     {
         int backwardIndex = inputBufferIndex-1;
-        while (inputBuffer[backwardIndex] == ' ' || 
-               inputBuffer[backwardIndex] == '\t') 
+        while (isSpace(inputBuffer[backwardIndex])) 
         { 
             --backwardIndex; //backward rolling
         } 
@@ -730,25 +850,34 @@ void processTextNode()
         char nextChar = readNextChar();
         if (isLineBreak(nextChar))
         {
-            readWhites();
             if (options->oneLineText)
             { 
+                readWhites(TRUE);
+              
                 //as we can put text on one line, remove the line break 
                 //and replace it by a space but only if the previous 
                 //char wasn't a space
-                if (getPreviousInsertedChar() != ' ') 
-                { 
-                    putCharInBuffer(' '); 
-                }
+                if (getPreviousInsertedChar() != ' ') { putCharInBuffer(' '); }
             }
-            else 
+            else if (options->alignText)
             {
+                int read = readWhites(FALSE);
+                if (nextChar == '\r' && read == 0 && getNextChar() == '\n') //handles the '\r\n'
+                {
+                   nextChar = readNextChar();
+                   readWhites(FALSE);
+                }
+              
                 //put a new line only if the closing tag is not reached
                 if (getNextChar() != '<') 
                 {   
                     putNewLine(); 
                 } 
             }
+            else
+            {
+                putCharInBuffer(nextChar);
+            }
         }
         else
         {
@@ -773,15 +902,16 @@ void processTextNode()
     lastNodeOpen = FALSE;
 }
 
-void processCDATA()
+void processCDATA(void)
 {
-    gboolean inlineAllowed = FALSE;
+    bool inlineAllowed = FALSE;
     if (options->inlineCdata) { inlineAllowed = isInlineNodeAllowed(); }
+    if (inlineAllowed && !options->oneLineCdata) { inlineAllowed = isOnSingleLine(9, ']', ']'); }
     if (inlineAllowed) { resetBackwardIndentation(TRUE); }
     
     putNextCharsInBuffer(9); //putting the '<![CDATA[' into the buffer
     
-    gboolean loop = TRUE;
+    bool loop = TRUE;
     char oldChar = '[';
     while(loop)
     {
@@ -791,24 +921,93 @@ void processCDATA()
         
         if (!isLineBreak(nextChar)) //the cdata simply continues
         {
-            putCharInBuffer(nextChar);
-            oldChar = nextChar;
-        }
-        else if (!options->oneLineCdata)
-        {
-            readWhites(); //strip the whites and new line
-            putNewLine(); //put a new indentation line
-            oldChar = ' '; //and update the last char
+            if (options->oneLineCdata && isSpace(nextChar))
+            {
+                //removes all the unecessary spaces
+                while(isSpace(nextChar2))
+                {
+                    nextChar = readNextChar();
+                    nextChar2 = getNextChar();
+                }
+                
+                putCharInBuffer(' ');
+                oldChar = ' ';
+            }
+            else
+            {
+                //comment is left untouched
+                putCharInBuffer(nextChar);
+                oldChar = nextChar;
+            }
             
-            //TODO manage relative spacing
+            if (!loop && options->alignCdata) //end of cdata
+            {
+                //ensures the chars preceding the first '-' are all spaces (there are at least
+                //10 spaces in front of the ']]>' for the alignment with '<![CDATA[')
+                bool onlySpaces = xmlPrettyPrinted[xmlPrettyPrintedIndex-3] == ' ' &&
+                                  xmlPrettyPrinted[xmlPrettyPrintedIndex-4] == ' ' &&
+                                  xmlPrettyPrinted[xmlPrettyPrintedIndex-5] == ' ' &&
+                                  xmlPrettyPrinted[xmlPrettyPrintedIndex-6] == ' ' &&
+                                  xmlPrettyPrinted[xmlPrettyPrintedIndex-7] == ' ' &&
+                                  xmlPrettyPrinted[xmlPrettyPrintedIndex-8] == ' ' &&
+                                  xmlPrettyPrinted[xmlPrettyPrintedIndex-9] == ' ' &&
+                                  xmlPrettyPrinted[xmlPrettyPrintedIndex-10] == ' ' &&
+                                  xmlPrettyPrinted[xmlPrettyPrintedIndex-11] == ' ';
+                
+                //if all the preceding chars are white, then go for replacement
+                if (onlySpaces)
+                {
+                    xmlPrettyPrintedIndex -= 11; //remove indentation spaces
+                    putCharsInBuffer("]]"); //reset the first chars of '-->'
+                }
+            }
+        }
+        else if (!options->oneLineCdata && !inlineAllowed) //line break
+        {
+            //if the cdata need to be aligned, just add 9 spaces
+            if (options->alignCdata) 
+            {
+                int read = readWhites(FALSE); //strip the whites and new line
+                if (nextChar == '\r' && read == 0 && getNextChar() == '\n') //handles the \r\n return line
+                {
+                    readNextChar(); 
+                    readWhites(FALSE);
+                }
+              
+                putNewLine(); //put a new indentation line
+                putCharsInBuffer("         "); //align with <![CDATA[
+                oldChar = ' '; //and update the last char
+            }
+            else
+            {
+                putCharInBuffer(nextChar);
+                oldChar = nextChar;
+            }
         }
         else //cdata are inlined
         {
-            readWhites(); //strip the whites and add a space if necessary
-            if(getPreviousInsertedChar() != ' ') { putCharInBuffer(' '); }
+            readWhites(TRUE); //strip the whites and add a space if necessary
+            if(getPreviousInsertedChar() != ' ' &&
+               strncmp(xmlPrettyPrinted+xmlPrettyPrintedIndex-9, "<![CDATA[", 9) != 0) //prevents adding a space at the beginning 
+            { 
+                putCharInBuffer(' '); 
+                oldChar = ' ';
+            }
         }
     }
     
+    //if the cdata is inline, then all the trailing spaces are removed
+    if (options->oneLineCdata)
+    {
+        xmlPrettyPrintedIndex -= 2; //because of the last ']]' inserted
+        while(isWhite(xmlPrettyPrinted[xmlPrettyPrintedIndex-1]))
+        {
+            --xmlPrettyPrintedIndex;
+        }
+        putCharsInBuffer("]]");
+    }
+    
+    //finalize the cdata
     char lastChar = readNextChar(); //should be '>'
     if (lastChar != '>') 
     { 
@@ -821,18 +1020,18 @@ void processCDATA()
     
     if (inlineAllowed) { appendIndentation = FALSE; }
     
-    //there vas no node open
+    //there was no node open
     lastNodeOpen = FALSE;
 }
 
-void processDoctype()   
+void processDoctype(void)
 {
     putNextCharsInBuffer(9); //put the '<!DOCTYPE' into the buffer
     
-    gboolean loop = TRUE;
+    bool loop = TRUE;
     while(loop)
     {
-        readWhites();
+        readWhites(TRUE);
         putCharInBuffer(' '); //only one space for the attributes
         
         int nextChar = readNextChar();
@@ -886,30 +1085,42 @@ void processDoctype()
     }
 }
 
-void processDoctypeElement()
+void processDoctypeElement(void)
 {
     printError("ELEMENT is currently not supported by PrettyPrinter\n");
     result = PRETTY_PRINTING_NOT_SUPPORTED_YET;
 }
 
-void printError(char *msg, ...) 
+void printError(const char *msg, ...)
 {
     va_list va;
     va_start(va, msg);
-    g_warning(msg, va);
+    #ifdef HAVE_GLIB
+    g_logv(G_LOG_DOMAIN, G_LOG_LEVEL_WARNING, msg, va);
+    #else
+    vfprintf(stderr, msg, va);
+    putc('\n', stderr);
+    #endif
     va_end(va);
-    
-    //TODO also do a fprintf on stderr ?
-    
-    printDebugStatus();
-} 
 
-void printDebugStatus()
+    printDebugStatus();
+}
+
+void printDebugStatus(void)
 {
+    #ifdef HAVE_GLIB
     g_debug("\n===== INPUT =====\n%s\n=================\ninputLength = %d\ninputIndex = %d\noutputLength = %d\noutputIndex = %d\n", 
             inputBuffer, 
             inputBufferLength, 
             inputBufferIndex,
             xmlPrettyPrintedLength,
             xmlPrettyPrintedIndex);
+    #else
+    PP_ERROR("\n===== INPUT =====\n%s\n=================\ninputLength = %d\ninputIndex = %d\noutputLength = %d\noutputIndex = %d\n", 
+            inputBuffer, 
+            inputBufferLength, 
+            inputBufferIndex,
+            xmlPrettyPrintedLength,
+            xmlPrettyPrintedIndex);
+    #endif
 }
