@@ -19,6 +19,10 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "geany.h"
 #include "support.h"
 
@@ -101,10 +105,11 @@ static void shift_left_cb(G_GNUC_UNUSED GtkMenuItem *menuitem,
          }
 
       startline = sci_get_line_from_position(sci, startpos);
+      /* Setting also start point for 1st line */
+      linepos = sci_get_position_from_line(sci, startline);
       endline = sci_get_line_from_position(sci, endpos);
 
       /* normal mode */
-      // if(sci_get_selection_mode(sci) == 1){
       if(startline == endline){
 
          /* get the text in question */
@@ -248,6 +253,7 @@ static void shift_right_cb(G_GNUC_UNUSED GtkMenuItem *menuitem,
          }
 
       startline = sci_get_line_from_position(sci, startpos);
+      linepos = sci_get_position_from_line(sci, startline);
       endline = sci_get_line_from_position(sci, endpos);
 
       /* normal mode */
@@ -347,22 +353,22 @@ static void shift_right_cb(G_GNUC_UNUSED GtkMenuItem *menuitem,
 
 
 static void kb_shift_left(G_GNUC_UNUSED guint key_id){
-   
+
    /* sanity check */
    if (document_get_current() == NULL){
        return;
        }
-   
+
    shift_left_cb(NULL, NULL);
    }
 
 static void kb_shift_right(G_GNUC_UNUSED guint key_id){
-   
+
    /* sanity check */
    if (document_get_current() == NULL){
        return;
        }
-   
+
    shift_right_cb(NULL, NULL);
    }
 
@@ -384,7 +390,7 @@ void plugin_init(G_GNUC_UNUSED GeanyData *data){
        menu_item_shift_right);
    g_signal_connect(menu_item_shift_right, "activate",
        G_CALLBACK(shift_right_cb), NULL);
-   
+
    /* make sure our menu items aren't called when there is no doc open */
    ui_add_document_sensitive(menu_item_shift_right);
    ui_add_document_sensitive(menu_item_shift_left);
