@@ -4,6 +4,10 @@
  *      Copyright 2010 Adrian Dimitrov <dimitrov.adrian@gmail.com>
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <string.h>
@@ -18,6 +22,9 @@
 # include <gio/gio.h>
 #endif
 
+#ifdef G_OS_WIN32
+# include <windows.h>
+#endif
 
 /* These items are set by Geany before plugin_init() is called. */
 GeanyPlugin 				*geany_plugin;
@@ -50,7 +57,11 @@ static gboolean 			flag_on_expand_refresh 		= FALSE;
  * ------------------ */
 
 static gchar 				*CONFIG_FILE 				= NULL;
+#ifdef G_OS_WIN32
 static gchar 				*CONFIG_OPEN_EXTERNAL_CMD 	= "nautilus '%d'";
+#else
+static gchar 				*CONFIG_OPEN_EXTERNAL_CMD 	= "explorer '%d'";
+#endif
 static gboolean 			CONFIG_REVERSE_FILTER 		= FALSE;
 static gboolean 			CONFIG_ONE_CLICK_CHDOC 		= FALSE;
 static gboolean 			CONFIG_SHOW_HIDDEN_FILES 	= FALSE;
@@ -156,7 +167,6 @@ PluginCallback plugin_callbacks[] =
  * TREEBROWSER CORE FUNCTIONS
  * ------------------ */
 
-
 static gboolean
 tree_view_row_expanded_iter(GtkTreeView *tree_view, GtkTreeIter *iter)
 {
@@ -216,13 +226,52 @@ utils_pixbuf_from_path(gchar *path)
 #endif
 }
 
+
+/* result must be freed */
+static gchar*
+path_is_in_dir(gchar* src, gchar* find)
+{
+	int i = 0;
+
+	gchar *diffed_path = NULL, *tmp = NULL;
+	gchar **src_segments = NULL, **find_segments = NULL;
+	guint src_segments_n = 0, find_segments_n = 0, n = 0;
+
+	src_segments = g_strsplit(src, G_DIR_SEPARATOR_S, 0);
+	find_segments = g_strsplit(find, G_DIR_SEPARATOR_S, 0);
+
+	src_segments_n = g_strv_length(src_segments);
+	find_segments_n = g_strv_length(find_segments);
+
+	n = src_segments_n;
+	if (find_segments_n < n)
+		n = find_segments_n;
+
+	for(i = 1; i<n; i++)
+		if (utils_str_equal(find_segments[i], src_segments[i]) != TRUE)
+			break;
+		else
+		{
+			tmp = g_strconcat(diffed_path == NULL ? "" : diffed_path,
+								G_DIR_SEPARATOR_S, find_segments[i], NULL);
+			g_free(diffed_path);
+			diffed_path = tmp;
+		}
+
+	g_strfreev(src_segments);
+	g_strfreev(find_segments);
+
+	return diffed_path;
+}
+
+/* Return: FALSE - if file is filtered and not shown, and TRUE - if file isn`t filtered, and have to be shown */
 static gboolean
 check_filtered(const gchar *base_name)
 {
 	gchar		**filters;
 	guint 		i;
 	gboolean 	temporary_reverse 	= FALSE;
-	const gchar *exts[] 			= {".o", ".obj", ".so", ".dll", ".a", ".lib"};
+	const gchar *exts[] 			= {".o", ".obj", ".so", ".dll", ".a", ".lib", ".la", ".lo", ".pyc"};
 	guint exts_len;
 	const gchar *ext;
 	gboolean	filtered;
@@ -265,64 +314,44 @@ check_filtered(const gchar *base_name)
 	return filtered;
 }
 
-static gboolean
-check_hidden(const gchar *uri)
-{
-	gboolean is_visible = TRUE;
-	gchar *base_name;
-
 #ifdef G_OS_WIN32
-# ifdef HAVE_GIO
-	GFile *file;
-	GFileInfo *info;
-
-	if (CONFIG_SHOW_HIDDEN_FILES)
-	{
-		g_object_unref(info);
-		g_object_unref(file);
-		g_free(base_name);
+static gboolean
+win32_check_hidden(const gchar *filename)
+{
+	DWORD attrs;
+	static wchar_t w_filename[MAX_PATH];
+	MultiByteToWideChar(CP_UTF8, 0, filename, -1, w_filename, sizeof(w_filename));
+	attrs = GetFileAttributesW(w_filename);
+	if (attrs != INVALID_FILE_ATTRIBUTES && attrs & FILE_ATTRIBUTE_HIDDEN)
 		return TRUE;
-	}
-
-	if (uri[strlen(uri) - 1] == '~')
-	{
-		g_object_unref(info);
-		g_object_unref(file);
-		g_free(base_name);
-		return FALSE;
-	}
-
-	file = g_file_new_for_path(uri);
-	info = g_file_query_info(file, G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN, 0, NULL, NULL);
-	if (info)
-	{
-		if (g_file_info_get_is_hidden(info))
-			is_visible = FALSE;
-		g_object_unref(info);
-	}
-	g_object_unref(file);
-# endif /* HAVE_GIO */
-#else /* G_OS_WIN32 */
-
-	if (CONFIG_SHOW_HIDDEN_FILES)
-	{
-		g_free(base_name);
-		return TRUE;
-	}
-
-	if (uri[strlen(uri) - 1] == '~')
-	{
-		g_free(base_name);
-		return FALSE;
-	}
-
-	base_name = g_path_get_basename(uri);
-	if (base_name[0] == '.')
-		is_visible = FALSE;
-	g_free(base_name);
+	return FALSE;
+}
 #endif
 
-	return is_visible;
+/* Returns: whether name should be hidden. */
+static gboolean
+check_hidden(const gchar *filename)
+{
+	const gchar *base_name = NULL;
+	base_name = g_path_get_basename(filename);
+	gsize len;
+
+	if (! NZV(base_name))
+		return FALSE;
+
+#ifdef G_OS_WIN32
+	if (win32_check_hidden(filename))
+		return TRUE;
+#else
+	if (base_name[0] == '.')
+		return TRUE;
+#endif
+
+	len = strlen(base_name);
+	if (base_name[len - 1] == '~')
+		return TRUE;
+
+	return FALSE;
 }
 
 static gchar*
@@ -359,12 +388,15 @@ static gchar *
 get_terminal()
 {
 	gchar 		*terminal;
+#ifdef G_OS_WIN32
+	terminal = g_strdup("cmd");
+#else
 	const gchar *term = g_getenv("TERM");
-
 	if (term != NULL)
 		terminal = g_strdup(term);
 	else
 		terminal = g_strdup("xterm");
+#endif
 	return terminal;
 }
 
@@ -399,13 +431,13 @@ treebrowser_checkdir(gchar *directory)
 static void
 treebrowser_chroot(gchar *directory)
 {
-	if (g_str_has_suffix(directory, "/"))
+	if (g_str_has_suffix(directory, G_DIR_SEPARATOR_S))
 		g_strlcpy(directory, directory, strlen(directory));
 
 	gtk_entry_set_text(GTK_ENTRY(addressbar), directory);
 
 	if (!directory || strlen(directory) == 0)
-		directory = "/";
+		directory = G_DIR_SEPARATOR_S;
 
 	if (! treebrowser_checkdir(directory))
 		return;
@@ -460,7 +492,7 @@ treebrowser_browse(gchar *directory, gpointer parent)
 			is_dir 		= g_file_test (uri, G_FILE_TEST_IS_DIR);
 			utf8_name 	= utils_get_utf8_from_locale(fname);
 
-			if (check_hidden(uri))
+			if (!check_hidden(uri))
 			{
 				GdkPixbuf *icon = NULL;
 
@@ -748,26 +780,78 @@ gtk_tree_store_iter_clear_nodes(gpointer iter, gboolean delete_root)
 		gtk_tree_store_remove(GTK_TREE_STORE(treestore), iter);
 }
 
+static gboolean
+treebrowser_expand_to_path(gchar* root, gchar* find)
+{
+	int i = 0, j = 0;
+	gboolean founded = FALSE, global_founded = FALSE;
+	gchar *new = "";
+	gchar **root_segments = NULL, **find_segments = NULL;
+	guint root_segments_n = 0, find_segments_n = 0;
+
+	root_segments = g_strsplit(root, G_DIR_SEPARATOR_S, 0);
+	find_segments = g_strsplit(find, G_DIR_SEPARATOR_S, 0);
+
+	root_segments_n = g_strv_length(root_segments)-1;
+	find_segments_n = g_strv_length(find_segments)-1;
+
+
+	for (i = 1; i<=find_segments_n; i++)
+	{
+		new = g_strconcat(new, G_DIR_SEPARATOR_S, find_segments[i], NULL);
+
+		if (founded)
+		{
+			if (treebrowser_search(new, NULL))
+				global_founded = TRUE;
+		}
+		else
+			if (utils_str_equal(root, new) == TRUE)
+				founded = TRUE;
+	}
+
+	g_free(new);
+	g_strfreev(root_segments);
+	g_strfreev(find_segments);
+
+	return global_founded;
+}
+
 static gboolean
 treebrowser_track_current()
 {
 
 	GeanyDocument	*doc 		= document_get_current();
 	gchar 			*path_current;
-	gchar			**path_segments;
+	gchar			**path_segments = NULL;
+	gchar 			*froot = NULL;
 
 	if (doc != NULL && doc->file_name != NULL && g_path_is_absolute(doc->file_name))
 	{
 		path_current = utils_get_locale_from_utf8(doc->file_name);
 
-		path_segments = g_strsplit(path_current, G_DIR_SEPARATOR_S, 0);
-
-		treebrowser_search(path_current, NULL);
 		/*
-		 * NEED TO REWORK THE CONCEPT
+		 * Checking if the document is in the expanded or collapsed files
 		 */
+		if (! treebrowser_search(path_current, NULL))
+		{
+			/*
+			 * Else we have to chroting to the document`s nearles path
+			 */
+
+			froot = path_is_in_dir(addressbar_last_address, g_path_get_dirname(path_current));
+
+			if (froot == NULL)
+				froot = g_strdup(G_DIR_SEPARATOR_S);
+
+			if (utils_str_equal(froot, addressbar_last_address) != TRUE)
+				treebrowser_chroot(froot);
+
+			treebrowser_expand_to_path(froot, path_current);
+		}
 
 		g_strfreev(path_segments);
+		g_free(froot);
 		g_free(path_current);
 
 		return FALSE;
@@ -892,6 +976,12 @@ on_menu_set_as_root(GtkMenuItem *menuitem, gchar *uri)
 		treebrowser_chroot(uri);
 }
 
+static void
+on_menu_find_in_files(GtkMenuItem *menuitem, gchar *uri)
+{
+	search_show_find_in_files_dialog(uri);
+}
+
 static void
 on_menu_create_new_object(GtkMenuItem *menuitem, gchar *type)
 {
@@ -1035,6 +1125,26 @@ on_menu_close(GtkMenuItem *menuitem, gchar *uri)
 		document_close(document_find_by_filename(uri));
 }
 
+static void
+on_menu_close_children(GtkMenuItem *menuitem, gchar *uri)
+{
+	guint i;
+	int uri_len = strlen(uri);
+	for (i=0; i < GEANY(documents_array)->len; i++)
+	{
+		if (documents[i]->is_valid)
+		{
+			/* the docuemnt filename shoudl always be longer than the uri when closing children
+			 * Compare the beginingin of the filename string to see if it matchs the uri*/
+			if (strlen(documents[i]->file_name) > uri_len )
+			{
+				if (strncmp(uri,documents[i]->file_name, uri_len)==0)
+					document_close(documents[i]);
+			}
+		}
+	}
+}
+
 static void
 on_menu_copy_uri(GtkMenuItem *menuitem, gchar *uri)
 {
@@ -1099,6 +1209,11 @@ create_popup_menu(gchar *name, gchar *uri)
 	gtk_container_add(GTK_CONTAINER(menu), item);
 	g_signal_connect(item, "activate", G_CALLBACK(on_menu_refresh), NULL);
 
+	item = ui_image_menu_item_new(GTK_STOCK_FIND, _("Find in Files"));
+	gtk_container_add(GTK_CONTAINER(menu), item);
+	g_signal_connect(item, "activate", G_CALLBACK(on_menu_find_in_files), uri);
+	gtk_widget_set_sensitive(item, is_dir);
+
 	item = gtk_separator_menu_item_new();
 	gtk_container_add(GTK_CONTAINER(menu), item);
 
@@ -1128,6 +1243,11 @@ create_popup_menu(gchar *name, gchar *uri)
 	g_signal_connect(item, "activate", G_CALLBACK(on_menu_close), uri);
 	gtk_widget_set_sensitive(item, is_document);
 
+	item = ui_image_menu_item_new(GTK_STOCK_CLOSE, g_strdup_printf(_("Close Child Documents ")));
+	gtk_container_add(GTK_CONTAINER(menu), item);
+	g_signal_connect(item, "activate", G_CALLBACK(on_menu_close_children), uri);
+	gtk_widget_set_sensitive(item, is_dir);
+
 	item = ui_image_menu_item_new(GTK_STOCK_COPY, _("Copy full path to clipboard"));
 	gtk_container_add(GTK_CONTAINER(menu), item);
 	g_signal_connect(item, "activate", G_CALLBACK(on_menu_copy_uri), uri);
