@@ -1,6 +1,6 @@
 /*
  *  
- *  Copyright (C) 2010  Colomban Wendling <ban@herbesfolles.org>
+ *  Copyright (C) 2010-2011  Colomban Wendling <ban@herbesfolles.org>
  *  
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 
 #include "gwh-utils.h"
 #include "gwh-settings.h"
+#include "gwh-keybindings.h"
 
 
 #if ! GTK_CHECK_VERSION (2, 18, 0)
@@ -40,7 +41,13 @@
 #  define gtk_widget_get_visible(w) \
   (GTK_WIDGET_VISIBLE (w))
 # endif /* defined (gtk_widget_get_visible) */
-#endif
+#endif /* GTK_CHECK_VERSION (2, 18, 0) */
+#if ! GTK_CHECK_VERSION (2, 20, 0)
+# ifndef gtk_widget_get_mapped
+#  define gtk_widget_get_mapped(w) \
+  (GTK_WIDGET_MAPPED ((w)))
+# endif /* defined (gtk_widget_get_mapped) */
+#endif /* GTK_CHECK_VERSION (2, 20, 0) */
 
 
 struct _GwhBrowserPrivate
@@ -178,6 +185,39 @@ on_settings_inspector_window_geometry_notify (GObject    *object,
   g_free (geometry);
 }
 
+static void
+on_settings_wm_windows_skip_taskbar_notify (GObject    *object,
+                                            GParamSpec *pspec,
+                                            GwhBrowser *self)
+{
+  gboolean skips_taskbar;
+  
+  g_object_get (object, pspec->name, &skips_taskbar, NULL);
+  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (self->priv->inspector_window),
+                                    skips_taskbar);
+}
+
+static void
+on_settings_wm_windows_type_notify (GObject    *object,
+                                    GParamSpec *pspec,
+                                    GwhBrowser *self)
+{
+  gint      type;
+  gboolean  remap = gtk_widget_get_mapped (self->priv->inspector_window);
+  
+  /* We need to remap the window if we change its type because it's not doable
+   * when mapped. So, hack around. */
+  
+  g_object_get (object, pspec->name, &type, NULL);
+  if (remap) {
+    gtk_widget_unmap (self->priv->inspector_window);
+  }
+  gtk_window_set_type_hint (GTK_WINDOW (self->priv->inspector_window), type);
+  if (remap) {
+    gtk_widget_map (self->priv->inspector_window);
+  }
+}
+
 /* web inspector events handling */
 
 #define INSPECTOR_DETACHED(self) \
@@ -186,6 +226,19 @@ on_settings_inspector_window_geometry_notify (GObject    *object,
 #define INSPECTOR_VISIBLE(self) \
   (gtk_widget_get_visible ((self)->priv->inspector_view))
 
+static void
+inspector_set_visible (GwhBrowser *self,
+                       gboolean    visible)
+{
+  if (visible != INSPECTOR_VISIBLE (self)) {
+    if (visible) {
+      webkit_web_inspector_show (self->priv->inspector);
+    } else {
+      webkit_web_inspector_close (self->priv->inspector);
+    }
+  }
+}
+
 static void
 inspector_hide_window (GwhBrowser *self)
 {
@@ -238,6 +291,7 @@ on_inspector_close_window (WebKitWebInspector *inspector,
   inspector_hide_window (self);
   gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (self->priv->item_inspector),
                                      FALSE);
+  gtk_widget_grab_focus (gtk_widget_get_toplevel (self->priv->web_view));
   
   return TRUE;
 }
@@ -279,11 +333,7 @@ static void
 on_item_inspector_toggled (GtkToggleToolButton *button,
                            GwhBrowser          *self)
 {
-  if (gtk_toggle_tool_button_get_active (button)) {
-    webkit_web_inspector_show (self->priv->inspector);
-  } else {
-    webkit_web_inspector_close (self->priv->inspector);
-  }
+  inspector_set_visible (self, gtk_toggle_tool_button_get_active (button));
 }
 
 static void
@@ -293,6 +343,17 @@ on_url_entry_activate (GtkEntry    *entry,
   gwh_browser_set_uri (self, gtk_entry_get_text (entry));
 }
 
+static void
+update_history (GwhBrowser *self)
+{
+  WebKitWebView  *web_view = WEBKIT_WEB_VIEW (self->priv->web_view);
+  
+  gtk_widget_set_sensitive (GTK_WIDGET (self->priv->item_prev),
+                            webkit_web_view_can_go_back (web_view));
+  gtk_widget_set_sensitive (GTK_WIDGET (self->priv->item_next),
+                            webkit_web_view_can_go_forward (web_view));
+}
+
 static void
 update_load_status (GwhBrowser *self)
 {
@@ -317,10 +378,7 @@ update_load_status (GwhBrowser *self)
   gtk_widget_set_sensitive (GTK_WIDGET (self->priv->item_cancel), loading);
   gtk_widget_set_visible   (GTK_WIDGET (self->priv->item_cancel), loading);
   
-  gtk_widget_set_sensitive (GTK_WIDGET (self->priv->item_prev),
-                            webkit_web_view_can_go_back (web_view));
-  gtk_widget_set_sensitive (GTK_WIDGET (self->priv->item_next),
-                            webkit_web_view_can_go_forward (web_view));
+  update_history (self);
 }
 
 static void
@@ -353,6 +411,7 @@ on_web_view_uri_notify (GObject    *object,
   uri = webkit_web_view_get_uri (WEBKIT_WEB_VIEW (self->priv->web_view));
   gtk_entry_set_text (GTK_ENTRY (self->priv->url_entry), uri);
   g_object_set (self->priv->settings, "browser-last-uri", uri, NULL);
+  update_history (self);
 }
 
 static void
@@ -664,7 +723,7 @@ gwh_browser_class_init (GwhBrowserClass *klass)
   g_object_class_install_property (object_class, PROP_INSPECTOR_TRANSIENT_FOR,
                                    g_param_spec_object ("inspector-transient-for",
                                                         "Inspector transient for",
-                                                        "The parent window of the inspector when detached.",
+                                                        "The parent window of the inspector when detached",
                                                         GTK_TYPE_WINDOW,
                                                         G_PARAM_READWRITE));
   g_object_class_install_property (object_class, PROP_URI,
@@ -757,11 +816,40 @@ create_toolbar (GwhBrowser *self)
   return toolbar;
 }
 
+static GtkWidget *
+create_inspector_window (GwhBrowser *self)
+{
+  gboolean skips_taskbar;
+  gboolean window_type;
+  
+  g_object_get (self->priv->settings,
+                "wm-secondary-windows-skip-taskbar", &skips_taskbar,
+                "wm-secondary-windows-type", &window_type,
+                NULL);
+  self->priv->inspector_window_x = self->priv->inspector_window_y = 0;
+  self->priv->inspector_window = g_object_new (GTK_TYPE_WINDOW,
+                                               "type", GTK_WINDOW_TOPLEVEL,
+                                               "skip-taskbar-hint", skips_taskbar,
+                                               "type-hint", window_type,
+                                               "title", _("Web inspector"),
+                                               NULL);
+  g_signal_connect (self->priv->inspector_window, "delete-event",
+                    G_CALLBACK (on_inspector_window_delete_event), self);
+  g_signal_connect (self->priv->settings, "notify::wm-secondary-windows-skip-taskbar",
+                    G_CALLBACK (on_settings_wm_windows_skip_taskbar_notify), self);
+  g_signal_connect (self->priv->settings, "notify::wm-secondary-windows-type",
+                    G_CALLBACK (on_settings_wm_windows_type_notify), self);
+  gtk_container_add (GTK_CONTAINER (self->priv->inspector_window),
+                     self->priv->inspector_view);
+  
+  return self->priv->inspector_window;
+}
+
 static void
 gwh_browser_init (GwhBrowser *self)
 {
-  GtkWidget *scrolled;
-  WebKitWebSettings *wkws;
+  GtkWidget          *scrolled;
+  WebKitWebSettings  *wkws;
   
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GWH_TYPE_BROWSER,
                                             GwhBrowserPrivate);
@@ -795,16 +883,7 @@ gwh_browser_init (GwhBrowser *self)
   gtk_container_add (GTK_CONTAINER (self->priv->inspector_view),
                      self->priv->inspector_web_view);
   
-  self->priv->inspector_window_x = self->priv->inspector_window_y = 0;
-  self->priv->inspector_window = g_object_new (GTK_TYPE_WINDOW,
-                                               "type", GTK_WINDOW_TOPLEVEL,
-                                               "skip-taskbar-hint", TRUE,
-                                               "title", _("Web inspector"),
-                                               NULL);
-  g_signal_connect (self->priv->inspector_window, "delete-event",
-                    G_CALLBACK (on_inspector_window_delete_event), self);
-  gtk_container_add (GTK_CONTAINER (self->priv->inspector_window),
-                     self->priv->inspector_view);
+  self->priv->inspector_window = create_inspector_window (self);
   
   g_signal_connect (self, "notify::orientation",
                     G_CALLBACK (on_orientation_notify), self);
@@ -821,8 +900,6 @@ gwh_browser_init (GwhBrowser *self)
   g_signal_connect (self->priv->inspector, "attach-window",
                     G_CALLBACK (on_inspector_attach_window), self);
   
-  g_signal_connect (G_OBJECT (self->priv->web_view), "notify::load-status",
-                    G_CALLBACK (on_web_view_load_status_notify), self);
   g_signal_connect (G_OBJECT (self->priv->web_view), "notify::progress",
                     G_CALLBACK (on_web_view_progress_notify), self);
   g_signal_connect (G_OBJECT (self->priv->web_view), "notify::uri",
@@ -838,6 +915,11 @@ gwh_browser_init (GwhBrowser *self)
   g_signal_connect (G_OBJECT (self->priv->web_view), "scroll-event",
                     G_CALLBACK (on_web_view_scroll_event), self);
   
+  g_signal_connect (self->priv->web_view, "key-press-event",
+                    G_CALLBACK (gwh_keybindings_handle_event), self);
+  g_signal_connect (self->priv->inspector_view, "key-press-event",
+                    G_CALLBACK (gwh_keybindings_handle_event), self);
+  
   gtk_widget_grab_focus (self->priv->url_entry);
   
   g_signal_connect (self->priv->settings, "notify::browser-last-uri",
@@ -934,3 +1016,11 @@ gwh_browser_get_inspector_transient_for (GwhBrowser *self)
   
   return gtk_window_get_transient_for (GTK_WINDOW (self->priv->inspector_window));
 }
+
+void
+gwh_browser_toggle_inspector (GwhBrowser *self)
+{
+  g_return_if_fail (GWH_IS_BROWSER (self));
+  
+  inspector_set_visible (self, ! INSPECTOR_VISIBLE (self));
+}
