@@ -1,6 +1,6 @@
 /*
  *  
- *  Copyright (C) 2010  Colomban Wendling <ban@herbesfolles.org>
+ *  Copyright (C) 2010-2011  Colomban Wendling <ban@herbesfolles.org>
  *  
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,45 +19,19 @@
 
 #include "gwh-settings.h"
 
+#include "config.h"
+
 #include <string.h>
+#include <stdarg.h>
 #include <glib.h>
+#include <glib/gi18n-lib.h>
 #include <glib-object.h>
-#include <gtk/gtk.h> /* for GtkOrientation */
-
-#include "gwh-enum-types.h" /* for GwhUiBrowserPosition */
-
-
-/* sets @ptr to @value, freeing @ptr before.
- * note that:
- *  * ptr can be used to compute @value, assignation is done after comuptation
- *  * ptr is used twice, so it must be side-effects-free */
-#define setptr(ptr, value)       \
-  G_STMT_START {                 \
-    gpointer new_ptr = (value);  \
-    g_free (ptr);                \
-    ptr = new_ptr;               \
-  } G_STMT_END
+#include <gtk/gtk.h>
 
 
 struct _GwhSettingsPrivate
 {
-  gboolean              browser_auto_reload;
-  gchar                *browser_last_uri;
-  GtkOrientation        browser_orientation;
-  GwhBrowserPosition    browser_position;
-  gchar                *browser_separate_window_geometry;
-  gchar                *inspector_window_geometry;
-};
-
-enum
-{
-  PROP_0,
-  PROP_BROWSER_AUTO_RELOAD,
-  PROP_BROWSER_LAST_URI,
-  PROP_BROWSER_ORIENTATION,
-  PROP_BROWSER_POSITION,
-  PROP_BROWSER_SEPARATE_WINDOW_GEOMETRY,
-  PROP_INSPECTOR_WINDOW_GEOMETRY
+  GPtrArray *prop_array;
 };
 
 
@@ -72,28 +46,13 @@ gwh_settings_get_property (GObject    *object,
 {
   GwhSettings *self = GWH_SETTINGS (object);
   
-  switch (prop_id) {
-    case PROP_BROWSER_AUTO_RELOAD:
-      g_value_set_boolean (value, self->priv->browser_auto_reload);
-      break;
-    case PROP_BROWSER_LAST_URI:
-      g_value_set_string (value, self->priv->browser_last_uri);
-      break;
-    case PROP_BROWSER_ORIENTATION:
-      g_value_set_enum (value, self->priv->browser_orientation);
-      break;
-    case PROP_BROWSER_POSITION:
-      g_value_set_enum (value, self->priv->browser_position);
-      break;
-    case PROP_BROWSER_SEPARATE_WINDOW_GEOMETRY:
-      g_value_set_string (value, self->priv->browser_separate_window_geometry);
-      break;
-    case PROP_INSPECTOR_WINDOW_GEOMETRY:
-      g_value_set_string (value, self->priv->inspector_window_geometry);
-      break;
+  if (G_LIKELY (prop_id > 0 && prop_id <= self->priv->prop_array->len)) {
+    GValue *prop_value;
     
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+    prop_value = g_ptr_array_index (self->priv->prop_array, prop_id - 1);
+    g_value_copy (prop_value, value);
+  } else {
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
 }
 
@@ -105,29 +64,13 @@ gwh_settings_set_property (GObject      *object,
 {
   GwhSettings *self = GWH_SETTINGS (object);
   
-  switch (prop_id) {
-    case PROP_BROWSER_AUTO_RELOAD:
-      self->priv->browser_auto_reload = g_value_get_boolean (value);
-      break;
-    case PROP_BROWSER_LAST_URI:
-      setptr (self->priv->browser_last_uri, g_value_dup_string (value));
-      break;
-    case PROP_BROWSER_ORIENTATION:
-      self->priv->browser_orientation = g_value_get_enum (value);
-      break;
-    case PROP_BROWSER_POSITION:
-      self->priv->browser_position = g_value_get_enum (value);
-      break;
-    case PROP_BROWSER_SEPARATE_WINDOW_GEOMETRY:
-      setptr (self->priv->browser_separate_window_geometry,
-              g_value_dup_string (value));
-      break;
-    case PROP_INSPECTOR_WINDOW_GEOMETRY:
-      setptr (self->priv->inspector_window_geometry, g_value_dup_string (value));
-      break;
+  if (G_LIKELY (prop_id > 0 && prop_id <= self->priv->prop_array->len)) {
+    GValue *prop_value;
     
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+    prop_value = g_ptr_array_index (self->priv->prop_array, prop_id - 1);
+    g_value_copy (value, prop_value);
+  } else {
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
 }
 
@@ -137,7 +80,7 @@ gwh_settings_constructor (GType                  gtype,
                           GObjectConstructParam *properties)
 {
   static GObject *obj = NULL;
-
+  
   if (G_UNLIKELY (! obj)) {
     obj = G_OBJECT_CLASS (gwh_settings_parent_class)->constructor (gtype,
                                                                    n_properties,
@@ -152,16 +95,22 @@ gwh_settings_constructor (GType                  gtype,
   return g_object_ref (obj);
 }
 
+static void
+free_prop_item (gpointer data,
+                gpointer user_data)
+{
+  g_value_unset (data);
+  g_free (data);
+}
+
 static void
 gwh_settings_finalize (GObject *object)
 {
   GwhSettings *self = GWH_SETTINGS (object);
   
-  self->priv->browser_auto_reload = FALSE;
-  setptr (self->priv->browser_last_uri, NULL);
-  self->priv->browser_orientation = 0;
-  self->priv->browser_position = 0;
-  setptr (self->priv->inspector_window_geometry, NULL);
+  g_ptr_array_foreach (self->priv->prop_array, free_prop_item, NULL);
+  g_ptr_array_free (self->priv->prop_array, TRUE);
+  self->priv->prop_array = NULL;
   
   G_OBJECT_CLASS (gwh_settings_parent_class)->finalize (object);
 }
@@ -176,45 +125,6 @@ gwh_settings_class_init (GwhSettingsClass *klass)
   object_class->get_property  = gwh_settings_get_property;
   object_class->set_property  = gwh_settings_set_property;
   
-  g_object_class_install_property (object_class, PROP_BROWSER_AUTO_RELOAD,
-                                   g_param_spec_boolean ("browser-auto-reload",
-                                                         "Browser auto reload",
-                                                         "Whether the browser reloads itself upon document saving",
-                                                         TRUE,
-                                                         G_PARAM_READWRITE));
-  g_object_class_install_property (object_class, PROP_BROWSER_LAST_URI,
-                                   g_param_spec_string ("browser-last-uri",
-                                                        "Browser last URI",
-                                                        "Last URI visited by the browser",
-                                                        "about:blank",
-                                                        G_PARAM_READWRITE));
-  g_object_class_install_property (object_class, PROP_BROWSER_ORIENTATION,
-                                   g_param_spec_enum ("browser-orientation",
-                                                      "Browser orientation",
-                                                      "Orientation of the browser widget",
-                                                      GTK_TYPE_ORIENTATION,
-                                                      GTK_ORIENTATION_VERTICAL,
-                                                      G_PARAM_READWRITE));
-  g_object_class_install_property (object_class, PROP_BROWSER_POSITION,
-                                   g_param_spec_enum ("browser-position",
-                                                      "Browser position",
-                                                      "Position of the browser widget in Geany's UI",
-                                                      GWH_TYPE_BROWSER_POSITION,
-                                                      GWH_BROWSER_POSITION_MESSAGE_WINDOW,
-                                                      G_PARAM_READWRITE));
-  g_object_class_install_property (object_class, PROP_BROWSER_SEPARATE_WINDOW_GEOMETRY,
-                                   g_param_spec_string ("browser-separate-window-geometry",
-                                                        "Browser separate window geometry",
-                                                        "Last geometry of the separated browser's window",
-                                                        "400x300",
-                                                        G_PARAM_READWRITE));
-  g_object_class_install_property (object_class, PROP_INSPECTOR_WINDOW_GEOMETRY,
-                                   g_param_spec_string ("inspector-window-geometry",
-                                                        "Inspector window geometry",
-                                                        "Last geometry of the inspector window",
-                                                        "400x300",
-                                                        G_PARAM_READWRITE));
-  
   g_type_class_add_private (klass, sizeof (GwhSettingsPrivate));
 }
 
@@ -223,12 +133,7 @@ gwh_settings_init (GwhSettings *self)
 {
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GWH_TYPE_SETTINGS,
                                             GwhSettingsPrivate);
-  self->priv->browser_auto_reload       = TRUE;
-  self->priv->browser_last_uri          = g_strdup ("about:blank");
-  self->priv->browser_orientation       = GTK_ORIENTATION_VERTICAL;
-  self->priv->browser_position          = GWH_BROWSER_POSITION_MESSAGE_WINDOW;
-  self->priv->browser_separate_window_geometry = g_strdup ("400x300");
-  self->priv->inspector_window_geometry = g_strdup ("400x300");
+  self->priv->prop_array = g_ptr_array_new ();
 }
 
 
@@ -240,6 +145,86 @@ gwh_settings_get_default (void)
   return g_object_new (GWH_TYPE_SETTINGS, NULL);
 }
 
+static gboolean
+is_pspec_installed (GObject          *obj,
+                    const GParamSpec *pspec)
+{
+  GParamSpec  **pspecs;
+  guint         n_props;
+  guint         i;
+  gboolean      installed = FALSE;
+  
+  pspecs = g_object_class_list_properties (G_OBJECT_GET_CLASS (obj), &n_props);
+  for (i = 0; ! installed && i < n_props; i++) {
+    installed = (pspec->value_type == pspecs[i]->value_type &&
+                 strcmp (pspec->name, pspecs[i]->name) == 0);
+  }
+  g_free (pspecs);
+  
+  return installed;
+}
+
+void
+gwh_settings_install_property (GwhSettings *self,
+                               GParamSpec  *pspec)
+{
+  GValue *value;
+  
+  g_return_if_fail (GWH_IS_SETTINGS (self));
+  g_return_if_fail (G_IS_PARAM_SPEC (pspec));
+  
+  /* a bit hackish, but allows to install the same property twice because the
+   * class will not be destroyed if the plugin gets reloaded. safe since the
+   * object is a singleton that will never de destroyed, and the plugin is
+   * resident, so the object is still valid after a reload. */
+  if (is_pspec_installed (G_OBJECT (self), pspec)) {
+    return;
+  }
+  
+  value = g_value_init (g_malloc0 (sizeof *value), pspec->value_type);
+  switch (G_TYPE_FUNDAMENTAL (pspec->value_type)) {
+    #define HANDLE_BASIC_TYPE(NAME, Name, name)                                \
+      case G_TYPE_##NAME:                                                      \
+        g_value_set_##name (value, ((GParamSpec##Name*)pspec)->default_value); \
+        break;
+    
+    HANDLE_BASIC_TYPE (BOOLEAN, Boolean, boolean)
+    HANDLE_BASIC_TYPE (CHAR,    Char,    char)
+    HANDLE_BASIC_TYPE (UCHAR,   UChar,   uchar)
+    HANDLE_BASIC_TYPE (INT,     Int,     int)
+    HANDLE_BASIC_TYPE (UINT,    UInt,    uint)
+    HANDLE_BASIC_TYPE (LONG,    Long,    long)
+    HANDLE_BASIC_TYPE (ULONG,   ULong,   ulong)
+    HANDLE_BASIC_TYPE (INT64,   Int64,   int64)
+    HANDLE_BASIC_TYPE (UINT64,  UInt64,  uint64)
+    HANDLE_BASIC_TYPE (FLOAT,   Float,   float)
+    HANDLE_BASIC_TYPE (DOUBLE,  Double,  double)
+    HANDLE_BASIC_TYPE (ENUM,    Enum,    enum)
+    HANDLE_BASIC_TYPE (FLAGS,   Flags,   flags)
+    HANDLE_BASIC_TYPE (STRING,  String,  string)
+    
+    #undef HANDLE_BASIC_TYPE
+    
+    case G_TYPE_PARAM:
+    case G_TYPE_BOXED:
+    case G_TYPE_POINTER:
+    case G_TYPE_OBJECT:
+      /* nothing to do with these types, default GValue is fine since they have
+       * no default value */
+      break;
+    
+    default:
+      g_critical ("Unsupported property type \"%s\" for property \"%s\"",
+                  G_VALUE_TYPE_NAME (value), pspec->name);
+      g_value_unset (value);
+      g_free (value);
+      return;
+  }
+  g_ptr_array_add (self->priv->prop_array, value);
+  g_object_class_install_property (G_OBJECT_GET_CLASS (self),
+                                   self->priv->prop_array->len, pspec);
+}
+
 static void
 get_key_and_group_from_property_name (const gchar *name,
                                       gchar      **group,
@@ -347,6 +332,7 @@ gwh_settings_save_to_file (GwhSettings *self,
       success = FALSE;
     } else {
       success = g_file_set_contents (filename, data, length, error);
+      g_free (data);
     }
   }
   g_key_file_free (key_file);
@@ -473,11 +459,42 @@ gwh_settings_load_from_file (GwhSettings *self,
 
 
 
+/* display/edit widgets stuff */
+
+/*
+ * gwh_settings_widget_<type>_notify:
+ *   calls user's callback with appropriate arguments
+ * gwh_settings_widget_<type>_notify_callback:
+ *   a callback connected to a signal on the widget.
+ *   calls gwh_settings_widget_<type>_notify
+ * gwh_settings_widget_<type>_new:
+ *   creates the widget
+ * gwh_settings_widget_<type>_sync:
+ *   syncs widget's value with its setting
+ */
 
 
+typedef struct _GwhSettingsWidgetNotifyData
+{
+  GwhSettings  *settings;
+  GCallback     callback;
+  gpointer      user_data;
+} GwhSettingsWidgetNotifyData;
+
+
+static void
+gwh_settings_widget_boolean_notify (GObject                      *tbutton,
+                                    GwhSettingsWidgetNotifyData  *data)
+{
+  ((GwhSettingsWidgetBooleanNotify)data->callback) (data->settings,
+                                                    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tbutton)),
+                                                    data->user_data);
+}
+
+#define gwh_settings_widget_boolean_notify_callback gwh_settings_widget_boolean_notify
 
 static GtkWidget *
-gwh_settings_widget_new_boolean (GwhSettings   *self,
+gwh_settings_widget_boolean_new (GwhSettings   *self,
                                  const GValue  *value,
                                  GParamSpec    *pspec,
                                  gboolean      *needs_label)
@@ -493,17 +510,38 @@ gwh_settings_widget_new_boolean (GwhSettings   *self,
 }
 
 static void
-gwh_settings_widget_sync_boolean (GwhSettings *self,
-                                  GParamSpec  *pspec,
-                                  GtkWidget   *widget)
+gwh_settings_widget_boolean_sync (GwhSettings  *self,
+                                  GParamSpec   *pspec,
+                                  const GValue *old_value,
+                                  GtkWidget    *widget)
 {
-  g_object_set (self, pspec->name,
-                gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)),
-                NULL);
+  gboolean val = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
+  
+  if (g_value_get_boolean (old_value) != val) {
+    g_object_set (self, pspec->name, val, NULL);
+  }
 }
 
+static void
+gwh_settings_widget_enum_notify (GObject                     *object,
+                                 GwhSettingsWidgetNotifyData *data)
+{
+  GtkTreeIter   iter;
+  GtkComboBox  *cbox = GTK_COMBO_BOX (object);
+  
+  if (gtk_combo_box_get_active_iter (cbox, &iter)) {
+    gint val;
+    
+    gtk_tree_model_get (gtk_combo_box_get_model (cbox), &iter, 0, &val, -1);
+    ((GwhSettingsWidgetEnumNotify)data->callback) (data->settings, val,
+                                                   data->user_data);
+  }
+}
+
+#define gwh_settings_widget_enum_notify_callback gwh_settings_widget_enum_notify
+
 static GtkWidget *
-gwh_settings_widget_new_enum (GwhSettings  *self,
+gwh_settings_widget_enum_new (GwhSettings  *self,
                               const GValue *value,
                               GParamSpec   *pspec,
                               gboolean     *needs_label)
@@ -523,7 +561,7 @@ gwh_settings_widget_new_enum (GwhSettings  *self,
     gtk_list_store_append (store, &iter);
     gtk_list_store_set (store, &iter,
                         0, enum_class->values[i].value,
-                        1, enum_class->values[i].value_nick,
+                        1, _(enum_class->values[i].value_nick),
                         -1);
     if (g_value_get_enum (value) == enum_class->values[i].value) {
       active = i;
@@ -544,9 +582,10 @@ gwh_settings_widget_new_enum (GwhSettings  *self,
 }
 
 static void
-gwh_settings_widget_sync_enum (GwhSettings *self,
-                               GParamSpec  *pspec,
-                               GtkWidget   *widget)
+gwh_settings_widget_enum_sync (GwhSettings   *self,
+                               GParamSpec    *pspec,
+                               const GValue  *old_value,
+                               GtkWidget     *widget)
 {
   GtkTreeIter iter;
   
@@ -556,12 +595,75 @@ gwh_settings_widget_sync_enum (GwhSettings *self,
     
     model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
     gtk_tree_model_get (model, &iter, 0, &val, -1);
+    if (g_value_get_enum (old_value) != val) {
+      g_object_set (self, pspec->name, val, NULL);
+    }
+  }
+}
+
+static void
+gwh_settings_widget_int_notify (GObject                     *spin,
+                                GwhSettingsWidgetNotifyData *data)
+{
+  ((GwhSettingsWidgetIntNotify)data->callback) (data->settings,
+                                                gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin)),
+                                                data->user_data);
+}
+
+#define gwh_settings_widget_int_notify_callback gwh_settings_widget_int_notify
+
+static GtkWidget *
+gwh_settings_widget_int_new (GwhSettings  *self,
+                             const GValue *value,
+                             GParamSpec   *pspec,
+                             gboolean     *needs_label)
+{
+  GtkWidget      *button;
+  GtkObject      *adj;
+  GParamSpecInt  *pspec_int = G_PARAM_SPEC_INT (pspec);
+  
+  adj = gtk_adjustment_new ((gdouble)g_value_get_int (value),
+                            (gdouble)pspec_int->minimum,
+                            (gdouble)pspec_int->maximum,
+                            1.0, 10.0, 0.0);
+  button = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 0.0, 0);
+  *needs_label = TRUE;
+  
+  return button;
+}
+
+static void
+gwh_settings_widget_int_sync (GwhSettings  *self,
+                              GParamSpec   *pspec,
+                              const GValue *old_value,
+                              GtkWidget    *widget)
+{
+  gint val = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (widget));
+  
+  if (g_value_get_int (old_value) != val) {
     g_object_set (self, pspec->name, val, NULL);
   }
 }
 
+static void
+gwh_settings_widget_string_notify (GObject                     *entry,
+                                   GwhSettingsWidgetNotifyData *data)
+{
+  ((GwhSettingsWidgetStringNotify)data->callback) (data->settings,
+                                                   gtk_entry_get_text (GTK_ENTRY (entry)),
+                                                   data->user_data);
+}
+
+static void
+gwh_settings_widget_string_notify_callback (GObject                     *object,
+                                            GParamSpec                  *pspec,
+                                            GwhSettingsWidgetNotifyData *data)
+{
+  gwh_settings_widget_string_notify (object, data);
+}
+
 static GtkWidget *
-gwh_settings_widget_new_string (GwhSettings  *self,
+gwh_settings_widget_string_new (GwhSettings  *self,
                                 const GValue *value,
                                 GParamSpec   *pspec,
                                 gboolean     *needs_label)
@@ -576,21 +678,47 @@ gwh_settings_widget_new_string (GwhSettings  *self,
 }
 
 static void
-gwh_settings_widget_sync_string (GwhSettings *self,
-                                 GParamSpec  *pspec,
-                                 GtkWidget   *widget)
+gwh_settings_widget_string_sync (GwhSettings   *self,
+                                 GParamSpec    *pspec,
+                                 const GValue  *old_value,
+                                 GtkWidget     *widget)
 {
-  g_object_set (self, pspec->name, gtk_entry_get_text (GTK_ENTRY (widget)),
-                NULL);
+  const gchar *val      = gtk_entry_get_text (GTK_ENTRY (widget));
+  const gchar *old_val  = g_value_get_string (old_value);
+  
+  if (g_strcmp0 (old_val, val) != 0) {
+    g_object_set (self, pspec->name, val, NULL);
+  }
 }
 
 
 #define KEY_PSPEC   "gwh-settings-configure-pspec"
 #define KEY_WIDGET  "gwh-settings-configure-widget"
 
+/**
+ * gwh_settings_widget_new_full:
+ * @self: A GwhSettings object
+ * @prop_name: the name of the setting for which create a widget
+ * @setting_changed_callback: a callback to be called when the widget's value
+ *                            changes, or %NULL. The type of the callback
+ *                            is GwhSettingsWidget*Notify, depending of the
+ *                            setting type
+ * @user_data: user data to pass to @callback
+ * @notify_flags: notification flags
+ * 
+ * Creates a widgets to configure the value of a setting @prop_name.
+ * This setting will not be changed automatically, you'll need to call
+ * gwh_settings_widget_sync() when you want to synchronize the widget's value to
+ * the setting.
+ * 
+ * Returns: A #GtkWidget that displays and edit @prop_name.
+ */
 GtkWidget *
-gwh_settings_widget_new (GwhSettings *self,
-                         const gchar *prop_name)
+gwh_settings_widget_new_full (GwhSettings            *self,
+                              const gchar            *prop_name,
+                              GCallback               setting_changed_callback,
+                              gpointer                user_data,
+                              GwhSettingsNotifyFlags  notify_flags)
 {
   GtkWidget  *widget      = NULL;
   GParamSpec *pspec;
@@ -605,12 +733,29 @@ gwh_settings_widget_new (GwhSettings *self,
   g_value_init (&value, pspec->value_type);
   g_object_get_property (G_OBJECT (self), prop_name, &value);
   switch (G_TYPE_FUNDAMENTAL (G_VALUE_TYPE (&value))) {
-    #define HANDLE_TYPE(T, h) \
-      case T: widget = ((h) (self, &value, pspec, &needs_label)); break;
+    #define HANDLE_TYPE(T, t, signal)                                          \
+      case G_TYPE_##T:                                                         \
+        widget = gwh_settings_widget_##t##_new (self, &value, pspec,           \
+                                                &needs_label);                 \
+        if (setting_changed_callback) {                                        \
+          GwhSettingsWidgetNotifyData *data = g_malloc (sizeof *data);         \
+                                                                               \
+          data->settings  = self;                                              \
+          data->callback  = setting_changed_callback;                          \
+          data->user_data = user_data;                                         \
+          g_signal_connect_data (widget, signal,                               \
+                                 G_CALLBACK (gwh_settings_widget_##t##_notify_callback),\
+                                 data, (GClosureNotify)g_free, 0);             \
+          if (notify_flags & GWH_SETTINGS_NOTIFY_ON_CONNEXION) {               \
+            gwh_settings_widget_##t##_notify (G_OBJECT (widget), data);        \
+          }                                                                    \
+        }                                                                      \
+        break;
     
-    HANDLE_TYPE (G_TYPE_BOOLEAN,  gwh_settings_widget_new_boolean)
-    HANDLE_TYPE (G_TYPE_ENUM,     gwh_settings_widget_new_enum)
-    HANDLE_TYPE (G_TYPE_STRING,   gwh_settings_widget_new_string)
+    HANDLE_TYPE (BOOLEAN, boolean,  "toggled")
+    HANDLE_TYPE (ENUM,    enum,     "changed")
+    HANDLE_TYPE (INT,     int,      "value-changed")
+    HANDLE_TYPE (STRING,  string,   "notify::text")
     
     #undef HANDLE_TYPE
     
@@ -627,7 +772,7 @@ gwh_settings_widget_new (GwhSettings *self,
       gchar     *label;
       
       box = gtk_hbox_new (FALSE, 6);
-      label = g_strdup_printf ("%s:", g_param_spec_get_nick (pspec));
+      label = g_strdup_printf (_("%s:"), g_param_spec_get_nick (pspec));
       gtk_box_pack_start (GTK_BOX (box), gtk_label_new (label), FALSE, TRUE, 0);
       g_free (label);
       gtk_box_pack_start (GTK_BOX (box), widget, TRUE, TRUE, 0);
@@ -644,25 +789,38 @@ gwh_settings_widget_new (GwhSettings *self,
   return widget;
 }
 
-void
-gwh_settings_widget_sync (GwhSettings *self,
-                          GtkWidget   *widget)
+GtkWidget *
+gwh_settings_widget_new (GwhSettings *self,
+                         const gchar *prop_name)
+{
+  return gwh_settings_widget_new_full (self, prop_name, NULL, NULL, 0);
+}
+
+static gboolean
+gwh_settings_widget_sync_internal (GwhSettings *self,
+                                   GtkWidget   *widget)
 {
   GParamSpec *pspec;
+  GValue      value = {0};
   
-  g_return_if_fail (GWH_IS_SETTINGS (self));
-  g_return_if_fail (G_IS_OBJECT (widget));
+  g_return_val_if_fail (G_IS_OBJECT (widget), FALSE);
   
   widget = g_object_get_data (G_OBJECT (widget), KEY_WIDGET);
-  g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
   pspec = g_object_get_data (G_OBJECT (widget), KEY_PSPEC);
-  g_return_if_fail (G_IS_PARAM_SPEC (pspec));
+  g_return_val_if_fail (G_IS_PARAM_SPEC (pspec), FALSE);
+  g_value_init (&value, pspec->value_type);
+  g_object_get_property (G_OBJECT (self), pspec->name, &value);
   switch (G_TYPE_FUNDAMENTAL (pspec->value_type)) {
-    #define HANDLE_TYPE(T, h) case T: (h) (self, pspec, widget); break;
+    #define HANDLE_TYPE(T, t)                                                  \
+      case G_TYPE_##T:                                                         \
+        gwh_settings_widget_##t##_sync (self, pspec, &value, widget);          \
+        break;
     
-    HANDLE_TYPE (G_TYPE_BOOLEAN,  gwh_settings_widget_sync_boolean)
-    HANDLE_TYPE (G_TYPE_ENUM,     gwh_settings_widget_sync_enum)
-    HANDLE_TYPE (G_TYPE_STRING,   gwh_settings_widget_sync_string)
+    HANDLE_TYPE (BOOLEAN,  boolean)
+    HANDLE_TYPE (ENUM,     enum)
+    HANDLE_TYPE (INT,      int)
+    HANDLE_TYPE (STRING,   string)
     
     #undef HANDLE_TYPE
     
@@ -670,4 +828,44 @@ gwh_settings_widget_sync (GwhSettings *self,
       g_critical ("Unsupported property type \"%s\"",
                   g_type_name (pspec->value_type));
   }
+  g_value_unset (&value);
+  
+  return TRUE;
+}
+
+/**
+ * gwh_settings_widget_sync_v:
+ * @self: A #GwhSettings object
+ * @...: a %NULL-terminated list of widgets to sync
+ * 
+ * Same as gwh_settings_widget_sync() but emits notifications only after all
+ * widgets got synchronized.
+ */
+void
+gwh_settings_widget_sync_v (GwhSettings *self,
+                            ...)
+{
+  GtkWidget  *widget;
+  va_list     ap;
+  
+  g_return_if_fail (GWH_IS_SETTINGS (self));
+  
+  g_object_freeze_notify (G_OBJECT (self));
+  va_start (ap, self);
+  while ((widget = va_arg (ap, GtkWidget*))) {
+    if (! gwh_settings_widget_sync_internal (self, widget)) {
+      break;
+    }
+  }
+  va_end (ap);
+  g_object_thaw_notify (G_OBJECT (self));
+}
+
+void
+gwh_settings_widget_sync (GwhSettings *self,
+                          GtkWidget   *widget)
+{
+  g_return_if_fail (GWH_IS_SETTINGS (self));
+  
+  gwh_settings_widget_sync_internal (self, widget);
 }
